@@ -3,15 +3,15 @@ Application class to store properties of the application being created
 
 @copyright: 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 @copyright: 2012-2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 
-import os, sys, random, re, logging, math
+import os, sys, random, re, logging, time
 import wx
 
-import common, config, misc, plugins, errors, compat
+import common, config, misc, plugins, compat
 import bugdialog
 import new_properties as np
 
@@ -19,17 +19,17 @@ import new_properties as np
 
 class FileDirDialog(object):
     """Custom class which displays a FileDialog or a DirDialog, according to the value of the
-    L{Application.multiple_files} of its parent (instance of L{Application}).
+    Application.multiple_files of its parent (instance of Application).
 
-    @ivar default_extension: The default extension will be added to all file names without extension.
-    @ivar file_message: Message to show on the file dialog
-    @ivar dir_message: Message to show on the directory dialog
+    default_extension: The default extension will be added to all file names without extension.
+    file_message: Message to show on the file dialog
+    dir_message: Message to show on the directory dialog
 
-    @ivar file_style: Style for the file dialog
-    @ivar dir_style:  Style for the directory dialog
-    @ivar value:      Value returned by file or directory dialog on success
-    @ivar parent:     Parent instance of L{Application}
-    @ivar prev_dir:   Previous directory"""
+    file_style: Style for the file dialog
+    dir_style:  Style for the directory dialog
+    value:      Value returned by file or directory dialog on success
+    parent:     Parent instance of Application
+    prev_dir:   Previous directory"""
 
     def __init__(self, parent, wildcard=_("All files|*"), file_message=_("Choose a file"),dir_message=None,file_style=0):
         self.prev_dir = config.preferences.codegen_path or ""
@@ -63,12 +63,145 @@ class FileDirDialog(object):
         return wx.ID_CANCEL
 
     def get_value(self):
-        "Return the dialog value returned during the last L{ShowModal()} call;  @see: L{value}"
+        "Return the dialog value returned during the last ShowModal() call."
         return self.value
 
 
+class EditRoot(np.PropertyOwner):
+    IS_ROOT = True
+    parent = None
 
-class Application(np.PropertyOwner):
+    # XXX move this to EditBase
+    def get_all_children(self):
+        return self.children
+
+    def _get_child_pos(self, child):
+        return self.children.index(child)
+
+    def has_ancestor(self, node):
+        return False
+
+    def get_path(self):
+        return self.name
+
+    def add_item(self, child, index=None):
+        # XXX index is always None at the moment
+        if index is None:
+            self.children.append(child)
+        else:
+            if len(self.children)<=index:
+                self.children += [None]*(index - len(self.children) + 1)
+            self.children[index] = child
+        if child.IS_TOPLEVEL_WINDOW:
+            self.add_top_window(child.name)
+
+    def remove_item(self, child, level=0, dummy=False):
+        if child.IS_TOPLEVEL_WINDOW:
+            self.remove_top_window(child.name)
+        self.children.remove(child)
+
+    def write(self, output, tabs=0):
+        """Writes the xml equivalent of this tree to the given output file.
+        This function writes unicode to the outfile."""
+        # XXX move this to application.Application
+        timestring = time.asctime()  if not config.testing else  'XXX XXX NN NN:NN:NN NNNN'
+        output.append( u'<?xml version="1.0"?>\n'
+                       u'<!-- generated by wxGlade %s on %s -->\n\n' % (config.version, timestring) )
+
+        attrs = ["name","class","language","top_window","encoding","use_gettext", "overwrite", "mark_blocks",
+                 "for_version","is_template","indent_amount"]
+        props = [self.properties[attr] for attr in attrs]
+        attrs = dict( (attr,prop.get_string_value()) for attr,prop in zip(attrs,props) if not prop.deactivated )
+        top_window_p = self.properties["top_window"]
+        if top_window_p.deactivated and top_window_p.value:
+            attrs["top_window"] = top_window_p.value
+        attrs["option"] = self.properties["multiple_files"].get_string_value()
+        attrs["indent_symbol"] = self.properties["indent_mode"].get_string_value()
+        attrs["path"] = self.properties["output_path"].get_string_value()
+        attrs['use_new_namespace'] = 1
+        # add a . to the file extensions
+        attrs["source_extension"] = '.' + self.properties["source_extension"].get_string_value()
+        attrs["header_extension"] = '.' + self.properties["header_extension"].get_string_value()
+
+        inner_xml = []
+
+        if self.is_template and getattr(self, 'template_data', None):
+            self.template_data.write(inner_xml, tabs+1)
+
+        for c in self.children:
+            c.write(inner_xml, tabs+1)
+
+        output.extend( common.format_xml_tag( u'application', inner_xml, is_xml=True, **attrs ) )
+
+    def find_widget_from_path(self, path):
+        path = path.split("/")
+        index = 1  # skip 'app'
+        w = self
+        for index in range(1, len(path)):
+            if path[index].startswith("SLOT "):
+                pos = int(path[index].split(" ")[1])
+                if pos<len(w.children) and w.children[pos].IS_SLOT:
+                    w = w.children[pos]
+                    continue
+            children = [c for c in w.get_all_children() if c.name==path[index]]
+            if not children: return None
+            w = children[0]
+        return w
+
+    def clear(self):
+        # delete all children; call common.root.new() or .init() afterwards
+        while self.children:
+            self.children[-1].recursive_remove(0)
+
+    def _get_parent_tooltip(self, index):
+        return None
+
+    def _get_tooltip_string(self):
+        return None
+
+    # use following to track toplevel windows?
+    def child_widget_created(self, child, level):
+        pass
+
+    def destroying_child_widget(self, child, index):
+        pass
+
+    def destroyed_child_widget(self):
+        pass
+
+
+class _DirDialog:
+    def __init__(self, parent, message, style):
+        self.parent = parent
+        self.message = message
+        self.style = style
+        self.value = None
+    def ShowModal(self):
+        self.value = wx.DirSelector( self.message, self.value, self.style )
+        if self.value:
+            return wx.ID_OK
+    def set_value(self, value):
+        self.value = value
+    def get_value(self):
+        return self.value
+
+
+class OutputPathProperty(np.FileNameProperty):
+    dir_message = _("Choose a directory")
+    def _create_dialog(self):
+        if not self.owner.multiple_files:
+            return np.FileNameProperty._create_dialog(self)
+        parent = self.text.GetTopLevelParent()
+        style = wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST | wx.DD_NEW_DIR_BUTTON
+        dlg = _DirDialog(parent, self.dir_message, style=style)
+        value = self.value
+        if not value or value.strip()=="." and self.owner.filename:
+            value = os.path.dirname(self.owner.filename)
+        dlg.set_value(value)
+        return dlg
+
+
+class Application(EditRoot):
     "Properties of the application being created"
 
     all_supported_versions = ['2.8', '3.0']  # Supported wx versions
@@ -108,13 +241,13 @@ class Application(np.PropertyOwner):
         _PROPERTY_HELP["output_path"] = "Output file or directory; double click label to show in Explorer"
     elif sys.platform=="darwin":
         _PROPERTY_HELP["output_path"] = "Output file or directory; double click label to show in Finder"
-    is_sizer = False
-    _is_toplevel = False
-    base = None
+
+    WX_CLASS = "wxApp"
+    IS_WINDOW = IS_SIZER = IS_TOPLEVEL = IS_SLOT = False
+    CHILDREN = None  # any number of toplevel windows etc.
 
     def __init__(self):
         np.PropertyOwner.__init__(self)
-        self._logger = logging.getLogger(self.__class__.__name__)
 
         self.__saved    = True  # raw value for self.saved property; if True, there are no changes to save
         self.__filename = None  # raw value for the self.filename property; Name of the output XML file
@@ -139,9 +272,9 @@ class Application(np.PropertyOwner):
         # C++ file extension
         self.source_extension = np.TextProperty('cpp')
         self.header_extension = np.TextProperty('h')
-        # output path
+        # output path: file or directory, depening on 'multiple_files'
         output_path = config.default_output_path  if self.multiple_files else  config.default_output_file
-        self.output_path = np.FileNameProperty(output_path, style=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)
+        self.output_path = OutputPathProperty(output_path, style=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)
         self._update_output_path('python')
 
         self.overwrite = np.InvCheckBoxProperty(config.default_overwrite)
@@ -171,6 +304,7 @@ class Application(np.PropertyOwner):
         self.generate_code = np.ActionButtonProperty(self.generate_code)
 
         self.widget = None  # always None, just to keep interface to Tree similar to other editors
+        self.children = []  # the toplevel windows
         self.node = None
 
     def set_for_version(self, value):
@@ -192,16 +326,6 @@ class Application(np.PropertyOwner):
     def get_output_path(self):
         return os.path.normpath(os.path.expanduser(self.output_path))
 
-    def _add_page(self, label, page, sizer):
-        "Add a page to properties notebook (L{self.notebook})"
-        page.SetAutoLayout(True)
-        page.SetSizer(sizer)
-        sizer.Layout()
-        sizer.Fit(page)
-        self.notebook.AddPage(page, label)
-        h = page.GetSize()[1]
-        page.SetScrollbars(1, 5, 1, int(math.ceil(h / 5.0)))
-
     def set_encoding(self, value):
         try:
             unicode('a', value)
@@ -251,36 +375,35 @@ class Application(np.PropertyOwner):
         self.properties["source_extension"].set_blocked(blocked)
         self.properties["header_extension"].set_blocked(blocked)
 
-
     # properties: saved and filename
     def _get_saved(self):
         return self.__saved
     def _set_saved(self, value):
         if self.__saved != value:
             self.__saved = value
-            t = common.app_tree.get_title().strip()
-            if not value:
-                common.app_tree.set_title('* ' + t)
-            else:
-                if t[0] == '*':
-                    common.app_tree.set_title(t[1:].strip())
+            if not config.use_gui: return
+            self._set_title()
     saved = property(_get_saved, _set_saved)
+
     def _get_filename(self):
         return self.__filename
     def _set_filename(self, value):
         if self.__filename != value:
             self.__filename = value
-            if self.__saved:
-                flag = ' '
-            else:
-                flag = '* '
-            if self.__filename is not None:
-                common.app_tree.set_title('%s(%s)' % (flag, self.__filename))
-            else:
-                common.app_tree.set_title(flag)
+            self._set_title()
     filename = property(_get_filename, _set_filename)
 
+    def _set_title(self):
+        if not config.use_gui: return
+        title = ["wxGlade: "]
+        if not self.__saved:
+            title.append( "* " )
+        if self.__filename:
+            title.append( "(%s)"%self.__filename )
+        common.main.SetTitle( "".join(title).strip() )
+
     # interface from tree ##############################################################################################
+    # handle top_window property choices
     def add_top_window(self, name):
         p = self.properties["top_window"]
         p.add_choice(name)
@@ -301,8 +424,17 @@ class Application(np.PropertyOwner):
         p.choices[p.choices.index(oldname)] = newname
         p.set_choices()
 
+    def _get_top_window(self):
+        # helper for wxGladeFrame
+        toplevel_name = self.top_window
+        toplevel = None
+        for c in self.children:
+            if c.name==toplevel_name:
+                toplevel = c
+        return toplevel
+
     ####################################################################################################################
-    def properties_changed(self, modified):
+    def _properties_changed(self, modified, actions):
         # ['encoding', 'output_path', 'class', 'name', 'multiple_files', 'language', 'top_window', 'use_gettext',
         # 'use_gettext', 'is_template', 'overwrite', 'indent_mode', 'indent_amount', 'for_version', 'source_extension',
         # 'header_extension']
@@ -318,6 +450,14 @@ class Application(np.PropertyOwner):
             if block:
                 self.properties["mark_blocks"].set(True)
             self.properties["mark_blocks"].set_blocked(block)
+        if modified and len(modified)==1 and "multiple_files" in modified:
+            # the user has just edited
+            p = self.properties["output_path"]
+            if self.multiple_files:
+                p.set(os.path.dirname(p.value) or ".")
+            else:
+                extension = common.code_writers[self.language].default_extensions[0]
+                p.set( os.path.join(p.value or "", "wxglade_out.%s"%extension) )
 
     def _init(self):
         # common part for init and new
@@ -353,12 +493,10 @@ class Application(np.PropertyOwner):
         self.properties_changed(None)
 
     def generate_code(self, preview=False, out_path=None, widget=None):
-        if config.use_gui:
-            common.property_panel.flush()
-        else:
-            np.flush_current_property()
+        np.flush_current_property()
         if out_path is None:
             out_path = os.path.expanduser(self.output_path.strip())
+            if not out_path and self.multiple_files: out_path = "."
             if not os.path.isabs(out_path) and (out_path and self.filename):
                 out_path = os.path.join(os.path.dirname(self.filename), out_path)
                 out_path = os.path.normpath(out_path)
@@ -379,23 +517,36 @@ class Application(np.PropertyOwner):
                                            "application." )
 
         if preview:
-            writer = common.code_writers["python"].copy()
+            writer = common.code_writers["preview"]
         else:
-            writer = common.code_writers[self.language]#.copy()
+            writer = common.code_writers[self.language]
+
+        error = writer.new_project(self, out_path, preview)
+        if error:
+            # prerequisites were checked and there is a problem
+            misc.error_message( _("Error generating code:\n%s")%error )
+            return
 
         try:
-            writer.new_project(self, out_path, preview)
-            writer.generate_code(self.node, widget)
+            writer.generate_code(self, widget)
             writer.finalize()
-        except errors.WxgBaseException as inst:
-            misc.error_message( _("Error generating code:\n%s")%inst )
-            return
         except EnvironmentError as inst:
-            bugdialog.ShowEnvironmentError(_('An IO/OS related error is occurred:'), inst)
+            bugdialog.ShowEnvironmentError(_('An IO related error has occurred:'), inst)
+            return
+        except UnicodeEncodeError as inst:
+            msg = _("Could not convert generated source code to encoding %s.\n"
+                    '(characters "%s")')
+            chars = inst.object[inst.start:inst.end] # .encode('unicode-escape')
+            msg = msg%(self.encoding, chars)
+            misc.error_message( msg )
+            return
+        except Exception as inst:
+            # unexpected / internal error
+            if config.testing or config.debugging: raise
             bugdialog.Show(_('Generate Code'), inst)
             return
         finally:
-            writer.clean_up(self.node)
+            writer.clean_up(widget or self)
 
         if preview or not config.use_gui: return
         if config.preferences.show_completion:
@@ -407,7 +558,6 @@ class Application(np.PropertyOwner):
             frame = app.GetTopWindow()
             frame.user_message(_('Code generated'))
 
-
     def is_visible(self):
         return True
 
@@ -424,6 +574,7 @@ class Application(np.PropertyOwner):
             else:
                 dirname, basename = os.path.split(self.filename)
                 basename, extension = os.path.splitext(basename)
+                basename = basename.replace(".", "_")
                 if not os.path.exists(dirname):
                     error = "Directory '%s' not found"%dirname
                 elif not os.path.isdir(dirname):
@@ -439,33 +590,21 @@ class Application(np.PropertyOwner):
     def preview(self, widget, position=None):
         """Generate and instantiate preview widget.
         None will be returned in case of errors. The error details are written to the application log file."""
-        # some checks
-        #if compat.IS_PHOENIX:
-            #found = common.app_tree.find_widgets_by_classnames(widget.node, "EditPropertyGridManager")
-            #if found:
-                #error = ("Preview with PropertyGridManager controls is currently deactivated as it causes crashes "
-                         #"with wxPython Phoenix")
-                #wx.MessageBox( error, _('Error'), wx.OK | wx.CENTRE | wx.ICON_EXCLAMATION )
-                #return
-        # XXX check other things as well, e.g. different bitmap sizes for BitmapButton
 
         preview_filename = self._get_preview_filename()
         if preview_filename is None: return
-        widget_class_name = widget.klass
-
-        # make a valid name for the class (this can be invalid for some sensible reasons...)
-        widget_class = widget.klass[widget.klass.rfind('.') + 1:]
-        widget_class = widget.klass[widget.klass.rfind(':') + 1:]
-        # ALB 2003-11-08: always randomize the class name: this is to make preview work even when there are multiple
-        # classes with the same name (which makes sense for XRC output...)
-        widget_class = '_%d_%s' % (random.randrange(10 ** 8, 10 ** 9), widget_class)
-        widget.properties["class"].set(widget_class)
 
         if wx.Platform == "__WXMAC__" and not compat.PYTHON2:
             # workaround for Mac OS testing: sometimes the caches need to be invalidated
             import importlib
             importlib.invalidate_caches()
 
+        # make a valid name for the class (this can be invalid for some sensible reasons...)
+        preview_classname = widget.WX_CLASS.split('.')[-1].split(':')[-1]
+        # randomize the class name to make preview work when there are multiple classes with the same name (e.g. XRC)
+        preview_classname = '_%d_%s' % (random.randrange(10 ** 8, 10 ** 9), preview_classname)
+        widget.properties["class"].set_temp(preview_classname)
+
         frame = None
         try:
             self.generate_code(True, preview_filename, widget)
@@ -481,12 +620,15 @@ class Application(np.PropertyOwner):
                 return None
 
             try:
-                preview_class = getattr(preview_module, widget.klass)
+                preview_class = getattr(preview_module, preview_classname) # .klass)
             except AttributeError:
-                # module loade previously -> do a re-load XXX this is required for Python 3; check alternatives
-                import importlib
-                preview_module = importlib.reload(preview_module)
-                preview_class = getattr(preview_module, widget.klass)
+                # module loaded previously -> do a re-load XXX this is required for Python 3; check alternatives
+                if sys.version_info.major<3:
+                    preview_module = reload(preview_module)
+                else:
+                    import importlib
+                    preview_module = importlib.reload(preview_module)
+                preview_class = getattr(preview_module, preview_classname)
 
             if not preview_class:
                 misc.error_message( _('No preview class "%s" found.\nThe details are written to the log file.\n'
@@ -495,13 +637,14 @@ class Application(np.PropertyOwner):
 
             if issubclass(preview_class, wx.MDIChildFrame):
                 frame = wx.MDIParentFrame(None, -1, '')
+                frame.SetMenuBar( wx.MenuBar() )  # avoid assertion error
                 child = preview_class(frame, -1, '')
                 child.SetTitle('<Preview> - ' + child.GetTitle())
                 w, h = child.GetSize()
                 frame.SetClientSize((w + 20, h + 20))
             elif not issubclass(preview_class, (wx.Frame,wx.Dialog)):
                 # the toplevel class isn't really toplevel, add a frame...
-                frame = wx.Frame(None, -1, widget_class_name)
+                frame = wx.Frame(None, -1, widget.klass)
                 if issubclass(preview_class, wx.MenuBar):
                     menubar = preview_class()
                     frame.SetMenuBar(menubar)
@@ -549,8 +692,7 @@ class Application(np.PropertyOwner):
             widget.preview_widget = None
             widget.properties["preview"].set_label(_('Show Preview'))
             bugdialog.Show(_("Generate Preview"), inst)
-        # XXX restore app state
-        widget.properties["class"].set(widget_class_name)
+
         return frame
     
     def on_char_hook(self, event):
@@ -569,7 +711,7 @@ class Application(np.PropertyOwner):
         if language is None:
             language = self.language
         if widget is not None:
-            if widget.__class__.__name__=="SizerSlot":
+            if widget.__class__.__name__ in ("SizerSlot", "Slot"):
                 return
             cname = common.class_names[widget.__class__.__name__]
             if language != 'XRC':
@@ -579,22 +721,22 @@ class Application(np.PropertyOwner):
                 xrcgen = common.code_writers['XRC']
                 ok = xrcgen.obj_builders.get(cname, None) is not xrcgen.NotImplementedXrcObject
             if not ok:
-                self._logger.warn( _('No %s code generator for %s (of type %s) available'),
+                logging.warn( _('No %s code generator for %s (of type %s) available'),
                                    misc.capitalize(language), widget.name, cname )
         else:
             # in this case, we check all the widgets in the tree
             def check_rec(node):
                 if node.widget is not None:
-                    self.check_codegen(node.widget)
+                    self.check_codegen(node)
                 if node.children:
                     for c in node.children:
                         check_rec(c)
-            if common.app_tree.root.children:
-                for c in common.app_tree.root.children:
+            if common.root.children:
+                for c in common.root.children:
                     check_rec(c)
 
     def _update_output_path(self, language):
-        "Update wildcards and default extension in the generic file and directory dialog (L{FileDirDialog})."
+        "Update wildcards and default extension in the generic file and directory dialog (FileDirDialog)."
 
         prop = self.properties["output_path"]
         prop.message = _("Select output file")  if self.multiple_files else  _("Select output directory")
@@ -610,8 +752,8 @@ class Application(np.PropertyOwner):
     def popup_menu(self, event, pos=None):
         # right click event -> expand all or show context menu
         expanded = True
-        for child_node in common.app_tree.root.children or []:
-            if not common.app_tree.IsExpanded(child_node.item) and child_node.children:
+        for child in self.children:
+            if not common.app_tree.IsExpanded(child.item) and child.children:
                 expanded = False
                 break
         if not expanded:
@@ -639,11 +781,20 @@ class Application(np.PropertyOwner):
         return (False, "Only toplevel widgets can be added here; click Frame or Dialog icon to do so")
 
     def check_compatibility(self, widget, typename=None):
-        if getattr(widget, "_is_toplevel", False) or typename=="window":
+        if (widget and widget.IS_TOPLEVEL) or typename=="window":
+            return (True, None)
+        if (widget and widget.WX_CLASS=="wxMenuBar") or typename=="menubar":
+            return (True, None)
+        if (widget and widget.WX_CLASS=="wxToolBar") or typename=="toolbar":
             return (True, None)
         return (False, "Only toplevel widgets can be pasted here (e.g. Frame or Dialog)")
 
     def clipboard_paste(self, clipboard_data):
         "Insert a widget from the clipboard to the current destination"
         import clipboard
-        return clipboard._paste(None, None, 0, clipboard_data)
+        return clipboard._paste(None, 0, clipboard_data)
+
+    ####################################################################################################################
+    def _label_editable(self):
+        return False
+
