@@ -1,18 +1,9 @@
 """\
 Lisp code generator
 
-How the code is generated: every time the end of an object is reached during
-the parsing of the xml tree, either the function 'add_object' or the function
-'add_class' is called: the latter when the object is a toplevel one, the former
-when it is not. In the last case, 'add_object' calls the appropriate ``writer''
-function for the specific object, found in the 'obj_builders' dict. Such
-function accepts one argument, the CodeObject representing the object for
-which the code has to be written, and returns 3 lists of strings, representing
-the lines to add to the '__init__', '__set_properties' and '__do_layout'
-methods of the parent object.
-
 @copyright: 2005 Surendra K Singhi <efuzzyone@users.sourceforge.net>
 @copyright: 2012-2016 Carsten Grohmann
+@copyright: 2019 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
@@ -20,8 +11,7 @@ import os
 import os.path
 import re
 
-from codegen import BaseLangCodeWriter, BaseSourceFileContent, BaseWidgetHandler
-import errors
+from codegen import BaseLangCodeWriter, BaseSourceFileContent
 import wcodegen
 
 
@@ -67,6 +57,7 @@ class SourceFileContent(BaseSourceFileContent):
         triple_quote_str = None
         tmp_in = self._load_file(self.name)
         out_lines = []
+        check_old_methods = []  # list of indices with set_properties or do_layout
         for line in tmp_in:
             quote_index = -1
             if not inside_triple_quote:
@@ -97,7 +88,7 @@ class SourceFileContent(BaseSourceFileContent):
                     self.new_classes_inserted = True
                 self.class_name = result.group(1)
                 self.class_name = self.format_classname(self.class_name)
-                self.classes[self.class_name] = 1  # add the found class to the list of classes of this module
+                self.classes.add( self.class_name )  # add the found class to the list of classes of this module
                 out_lines.append(line)
             elif not inside_block:
                 result = self.rec_block_start.match(line)
@@ -115,21 +106,20 @@ class SourceFileContent(BaseSourceFileContent):
                     if not self.class_name:
                         out_lines.append( '<%swxGlade replace %s>' % (self.nonce, which_block) )
                     else:
+                        if which_block in ("__do_layout","__set_properties"):
+                            # probably to be removed
+                            check_old_methods.append( len(out_lines) )
                         out_lines.append( '<%swxGlade replace %s %s>' % (self.nonce, which_class, which_block) )
                 else:
                     result = self.rec_event_handler.match(line)
                     if not inside_triple_quote and result:
                         which_handler = result.group('handler')
                         which_class = self.format_classname(result.group('class'))
-                        self.event_handlers.setdefault(
-                            which_class, {})[which_handler] = 1
+                        self.event_handlers.setdefault( which_class, set() ).add( which_handler )
                     if self.class_name and self.is_end_of_class(line):
                         # add extra event handlers here...
                         out_lines.append( '<%swxGlade event_handlers %s>' % (self.nonce, self.class_name) )
                     out_lines.append(line)
-                    if self.is_import_line(line):
-                        # add a tag to allow extra modules
-                        out_lines.append( '<%swxGlade extra_modules>\n' % self.nonce )
             else:
                 # ignore all the lines inside a wxGlade block
                 if self.rec_block_end.match(line):
@@ -138,18 +128,16 @@ class SourceFileContent(BaseSourceFileContent):
             # if we are here, the previous ``version'' of the file did not contain any class, so we must add the
             # new_classes tag at the end of the file
             out_lines.append('<%swxGlade insert new_classes>' % self.nonce)
+
+        # when moving from 0.9 to 1.0: remove empty methods "do_layout" and "set_properties"
+        while check_old_methods:
+            i = check_old_methods.pop(-1)
+            if out_lines[i+1].strip()==')':  # just end of block -> remove incl. trailing empty lines
+                self._remove_method(out_lines, i-1, i+1)
+
         # set the ``persistent'' content of the file
         self.content = out_lines
 
-    def is_import_line(self, line):
-        return line.startswith('(use-package :wx')
-
-
-
-class WidgetHandler(BaseWidgetHandler):
-    pass
-
-
 
 class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
     "Code writer class for writing Lisp code out of the designed GUI elements"
@@ -172,12 +160,6 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
     class_separator = '.'
     classattr_always = ['wxBoxSizer', 'wxStaticBoxSizer', 'wxGridSizer', 'wxFlexGridSizer']
 
-    #global_property_writers = {
-        #'font':            BaseLangCodeWriter.FontPropertyHandler,
-        #'events':          BaseLangCodeWriter.EventsPropertyHandler,
-        #'extraproperties': BaseLangCodeWriter.ExtraPropertiesPropertyHandler,
-        #}
-
     indent_level_func_body = 2
 
     name_ctor = '__init__'
@@ -186,19 +168,13 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
 
     SourceFileContent = SourceFileContent
 
-    tmpl_name_do_layout = '__do_layout'
-    tmpl_name_set_properties = '__set_properties'
+    tmpl_sizeritem = '(wxSizer_AddWindow (%s obj) (%s obj) %s %s %s nil)\n'  # will be overwritten and restored
 
     tmpl_cfunc_end = '%(tab)s)\n'
 
     tmpl_class_end = '\n%(comment)s end of class %(klass)s\n\n\n'
     tmpl_class_end_nomarker = '\n\n\n'
 
-    tmpl_func_do_layout = '\n' \
-                          '(defmethod do-layout ((obj %(klass)s))\n' \
-                          '%(content)s' \
-                          '%(tab)s)\n'
-
     tmpl_func_event_stub = """\
 (defun %(handler)s (function data event) ;;; wxGlade: %(klass)s.<event_handler>
 %(tab)s(print "Event handler '%(handler)s' not implemented!")
@@ -206,11 +182,6 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
 %(tab)s%(tab)s(wxEvent:wxEvent_Skip event)))
 """
 
-    tmpl_func_set_properties = '\n' \
-                          '(defmethod set-properties ((obj %(klass)s))\n' \
-                          '%(content)s' \
-                          '%(tab)s)\n'
-
     tmpl_func_empty = '%(tab)spass\n'
 
     tmpl_appfile = """\
@@ -285,7 +256,7 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
             ret = self.tmpl_detailed
         if ret:
             if not self._mark_blocks:
-                ret.replace(";;; end of class %(klass)s\n", "")
+                ret = ret.replace(";;; end of class %(klass)s\n", "")
             return ret
         
         # check for templates for simple startup code
@@ -293,6 +264,7 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
             return self.tmpl_gettext_simple
         elif not klass and not self._use_gettext:
             return self.tmpl_simple
+
     def init_lang(self, app_attrs):
         self.class_lines = []
 
@@ -302,21 +274,15 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
             """(ffi:default-foreign-language :stdc)\n\n""",
             ]
 
-        self.dependencies = {
-            '(use-package :wxCL)':         1,
-            '(use-package :wxFrame)':      1,
-            '(use-package :wx_main)':      1,
-            '(use-package :wx_wrapper)':   1,
-            '(use-package :wxWindow)':     1,
-            '(use-package :wxColour)':     1,
-            '(use-package :wxEvtHandler)': 1,
-            '(use-package :wxEvent)':      1,
-            }
+        self.dependencies = {'(use-package :wxCL)', '(use-package :wxFrame)', '(use-package :wx_main)',
+            '(use-package :wx_wrapper)', '(use-package :wxWindow)',
+            '(use-package :wxColour)', '(use-package :wxEvtHandler)', '(use-package :wxEvent)'}
 
     def check_values(self):
-        BaseLangCodeWriter.check_values(self)
-        if self.for_version > (2, 8):
-            raise errors.WxgLispWx3NotSupported("%d.%d" % self.for_version)
+        error = BaseLangCodeWriter.check_values(self)
+        if not error and self.for_version > (2, 8):
+            return "Generating Lisp code is only supported for wx version 2.8"
+        return None
 
     def add_app(self, app, top_win_class):
         top_win = app.top_window
@@ -327,84 +293,61 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         self.lang_mapping = { 'top_win': self._format_name(top_win), }
         BaseLangCodeWriter.add_app(self, app, top_win_class)
 
-    def add_object(self, sub_obj):
+    def add_object(self, parent_klass, parent, parent_builder, obj):
+        # get the widget builder instance
+
         # the lisp code gen add some hard coded depedencies
         # TODO: Move the hard coded dependencies to the widgets resp. sizers
 
-        # XXX temporarily disabled...
-        #sub_obj.name = self._format_name(sub_obj.name)
-        #sub_obj.parent.name = self._format_name(sub_obj.parent.name)
+        builder = self._get_object_builder(parent_klass, obj)
+        if not builder: return None
 
-        # get top level source code object and the widget builder instance
-        klass, builder = self._add_object_init(sub_obj)
-        if not klass or not builder:
-            return
-
-        if sub_obj.name not in ("spacer","sizerslot"):
-            self.class_lines.append( self._format_name(sub_obj.name) )
-        if (sub_obj.klass == "wxBoxSizer"  or sub_obj.klass == "wxStaticBoxSizer" or
-            sub_obj.klass == "wxGridSizer" or sub_obj.klass == "wxFlexGridSizer"):
-            self.dependencies['(use-package :wxSizer)'] = 1
+        if obj.IS_NAMED:
+            self.class_lines.append( self._format_name(obj.name) )
+        if obj.WX_CLASS in ("wxBoxSizer", "wxStaticBoxSizer", "wxGridSizer", "wxFlexGridSizer"):
+            self.dependencies.add( '(use-package :wxSizer)' )
         else:
-            if sub_obj.klass not in ("spacer", "sizerslot"):
-                key = '(use-package :%s)' % sub_obj.klass
-                self.dependencies[key] = 1
+            if obj.WX_CLASS not in ("spacer", "sizerslot"):
+                self.dependencies.add( '(use-package :%s)'%obj.get_instantiation_class() )
 
-        if sub_obj.klass == "wxMenuBar":
-            self.dependencies['(use-package :wxMenu)'] = 1
+        if obj.WX_CLASS == "wxMenuBar":
+            self.dependencies.add( '(use-package :wxMenu)' )
 
-        BaseLangCodeWriter.add_object(self, sub_obj)
-
-    def add_sizeritem(self, toplevel, sizer, obj, option, flag, border):
-        # XXX remove this hack
-        if obj.is_sizer:
-            self.tmpl_sizeritem = '(wxSizer_AddSizer (%s obj) (%s obj) %s %s %s nil)\n'
-        else:
-            self.tmpl_sizeritem = '(wxSizer_AddWindow (%s obj) (%s obj) %s %s %s nil)\n'
-
-        BaseLangCodeWriter.add_sizeritem( self, toplevel, sizer, obj, option, flag, border )
-
-    def add_spacer(self, toplevel, sizer, obj=None, proportion=0, flag='0', border=0):
-        # XXX remove this hack
-        self.tmpl_sizeritem = '(wxSizer_AddWindow (%s obj) (%s obj) %s %s %s nil)\n'
-        BaseLangCodeWriter.add_spacer(self, toplevel, sizer, obj, proportion, flag, border)
+        return BaseLangCodeWriter.add_object(self, parent_klass, parent, parent_builder, obj)
 
     def generate_code_background(self, obj):
-        self.dependencies['(use-package :wxColour)'] = 1
+        self.dependencies.add( '(use-package :wxColour)' )
         return BaseLangCodeWriter.generate_code_background(self, obj)
 
     def generate_code_ctor(self, code_obj, is_new, tab):
         code_lines = []
         write = code_lines.append
 
-        builder = self.obj_builders[code_obj.base]
+        builder = self.obj_builders[code_obj.WX_CLASS]
         mycn = getattr(builder, 'cn', self.cn)
         mycn_f = getattr(builder, 'cn_f', self.cn_f)
 
         # custom base classes support
-        custom_base = getattr( code_obj, 'custom_base', code_obj.properties.get('custom_base', None) )
-        if self.preview or (custom_base and not custom_base.strip()):
-            custom_base = None
+        custom_base = code_obj.check_prop_nodefault('custom_base') and code_obj.custom_base.strip() or None
 
         # generate constructor code
         if is_new:
-            base = mycn(code_obj.base)
-            klass = code_obj.klass
-            write('\n(defclass %s()\n' % klass)
+            base = mycn(code_obj.WX_CLASS)
+            write('\n(defclass %s()\n' % code_obj.klass)
             write(tab + "((top-window :initform nil :accessor slot-top-window)")
             for l in self.class_lines:
                 write("\n")
                 write(tab + "(" + l + " :initform nil :accessor slot-" + l + ")")
             write("))\n")
 
-            write("\n(defun make-%s ()\n" % klass)
-            write(tab + "(let ((obj (make-instance '%s)))\n" % klass)
+            write("\n(defun make-%s ()\n" % code_obj.klass)
+            write(tab + "(let ((obj (make-instance '%s)))\n" % code_obj.klass)
             write(tab + "  (init obj)\n")
             write(tab + "  (set-properties obj)\n")
             write(tab + "  (do-layout obj)\n")
             write(tab + "  obj))\n")
 
-            write('\n(defmethod init ((obj %s))\n' % klass)
+            write('\n(defmethod init ((obj %s))\n' % code_obj.klass)
             write("\"Method creates the objects contained in the class.\"\n")
 
         elif custom_base:
@@ -419,6 +362,11 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
                                             'function':self.name_ctor, 'klass':self.cn_class(code_obj.klass),
                                             'tab':tab} )
 
+        # the optional initial code from the code properties
+        if not self.preview and code_obj.check_prop("extracode_pre"):
+            for l in code_obj.properties["extracode_pre"].get_lines():
+                write(tab + l)
+
         style_p = code_obj.properties.get("style")
         if style_p and style_p.value_set != style_p.default_value:
             m_style = mycn_f( style_p.get_string_value() )
@@ -430,33 +378,32 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         if 'size' in code_obj.properties and code_obj.properties["size"].is_active():
             write( tab + self.generate_code_size(code_obj) )
 
-        # classes[code_obj.klass].deps now contains a mapping of child to
-        # parent for all children we processed...
-        object_order = []
-        for obj in self.classes[code_obj.klass].child_order:
-            # Don't add it again if already present
-            if obj in object_order:
-                continue
+        for l in builder.get_properties_code(code_obj):
+            write(tab + l)
 
-            object_order.append(obj)
+        if code_obj.check_prop_truth('extraproperties'):
+            for l in builder.generate_code_extraproperties(code_obj):
+                write(tab + l)
 
-            # Insert parent and ancestor objects before the current object
-            current_object = obj
-            for child, parent in self.classes[code_obj.klass].deps[:]:
-                if child is current_object:
-                    if parent not in object_order:
-                        idx = object_order.index(current_object)
-                        object_order.insert(idx, parent)
-                    current_object = parent
+        # the initial and final code for the contained elements
+        for l in self.classes[code_obj].init:
+            write(tab + l)
+        if self.classes[code_obj].final:
+            write(tab + "\n")
+            for l in self.classes[code_obj].final:
+                write(tab + l)
 
-                    # We processed the dependency: remove it
-                    self.classes[code_obj.klass].deps.remove((child, parent))
+        # now check if there is initial and final code for the element itself
+        for l in builder.get_init_code(code_obj):
+            write(tab+l)
 
-        # Write out the initialisation in the order we just generated
-        for obj in object_order:
-            if obj in self.classes[code_obj.klass].init_lines:
-                for l in self.classes[code_obj.klass].init_lines[obj]:
-                    write(tab + l)
+        for l in builder.get_layout_code(code_obj):
+            write(tab + l)
+
+        # the optional final code from the code properties
+        if not self.preview and code_obj.check_prop("extracode_post"):
+            for l in code_obj.properties["extracode_post"].get_lines():
+                write(tab + l)
 
         return code_lines
 
@@ -467,10 +414,8 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         if event_handlers:
             write('\n')
 
-        for win_id, event, handler, unused in event_handlers:
-            if win_id.startswith('#'):
-                win_id = win_id[1:]
-            win_id = self._format_name(win_id)
+        for obj, event, handler, unused in event_handlers:
+            win_id = self._format_name('obj.%s' % obj.name if not obj.IS_CLASS else 'obj')
 
             write( "%(tab)s(wxEvtHandler_Connect (slot-top-window obj) %(win_id)s (exp%(event)s)\n%(tab2)s" \
                    "(wxClosure_Create #'%(handler)s obj))\n" % 
@@ -479,25 +424,24 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         return code_lines
 
     def generate_code_font(self, obj):
-        self.dependencies['(use-package :wxFont)'] = 1
+        self.dependencies.add( '(use-package :wxFont)' )
         return BaseLangCodeWriter.generate_code_font(self, obj)
 
     def generate_code_foreground(self, obj):
-        self.dependencies['(use-package :wxColour)'] = 1
+        self.dependencies.add( '(use-package :wxColour)' )
         return BaseLangCodeWriter.generate_code_foreground(self, obj)
 
     def generate_code_id(self, obj, id=None):
         if id is None:
             id = obj.window_id
         if not id:
-            if obj is not None and "stockitem" in obj.properties and obj.stockitem:
+            if obj is not None and obj.check_prop_truth("stockitem"):
                 return '', self.cn("wxID_" + obj.stockitem)
             return '', self.cn('wxID_ANY')
         tokens = id.split('=', 1)
-        if len(tokens) == 2:
-            name, val = tokens
-        else:
+        if len(tokens) != 2:
             return '', self.cn(tokens[0])   # we assume name is declared elsewhere
+        name, val = tokens
         if not name:
             return '', self.cn(val)
         name = name.strip()
@@ -515,14 +459,11 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         objname = self.format_generic_access(obj)
         size = obj.properties["size"].get_string_value()
         use_dialog_units = (size[-1] == 'd')
-        if not obj.parent:
-            method = 'wxWindow_SetSize'
-        else:
-            method = 'SetMinSize'
+        method = 'SetMinSize'  if obj.parent_window else  'wxWindow_SetSize'
+
         if use_dialog_units:
             return '(%s %s(%s(%s (%s))))\n' % ( method, objname, self.cn('wxDLG_SZE'), objname, size[:-1] )
-        else:
-            return '%s.%s((%s))\n' % (objname, method, size)
+        return '%s.%s((%s))\n' % (objname, method, size)
 
     def _quote_str(self, s):
         if self._use_gettext:
@@ -530,9 +471,6 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         else:
             return '"%s"' % s
 
-    def add_object_format_name(self, name):
-        return '#obj.%s' % name
-
     def _format_classattr(self, obj):
         res = BaseLangCodeWriter._format_classattr(self, obj)
         if not res:
@@ -541,7 +479,7 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
             return obj.name
         # spacer.name is "<width>, <height>" already, but wxLisp expect
         # a tuple instead of two single values
-        elif obj.klass in ('spacer','sizerslot'):
+        elif obj.WX_CLASS in ('spacer','sizerslot'):
             return '(%s)' % obj.name
         # wxList use class attributes always (unfortunately)
 #        elif self.store_as_attr(obj):
@@ -550,11 +488,10 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         return 'slot-%s' % self._format_name(obj.name)
 
     def _format_import(self, klass):
-        stmt = '(require "%s")\n' % klass
-        return stmt
+        return '(require "%s")\n' % klass
 
     def _format_style(self, style, code_obj):
-        builder = self.obj_builders[code_obj.base]
+        builder = self.obj_builders[code_obj.WX_CLASS]
         mycn_f = getattr(builder, 'cn_f', self.cn_f)
 
         if not style: return ''
@@ -562,34 +499,31 @@ class LispCodeWriter(BaseLangCodeWriter, wcodegen.LispMixin):
         if not style: return ''
         style = style.strip().replace('.', '')
 
-        if code_obj.base == "wxFrame":
+        if code_obj.WX_CLASS == "wxFrame":
             stmt = '%%(tab)s(setf (slot-top-window obj) (wxFrame_create nil wxID_ANY \"\" -1 -1 -1 -1 %s))\n' % style
-        elif code_obj.base == "wxDialog":
+        elif code_obj.WX_CLASS == "wxDialog":
             stmt = '%%(tab)s(setf (slot-top-window obj) (wxDialog_create nil wxID_ANY \"\" -1 -1 -1 -1 %s))\n' % style
-            self.dependencies['(use-package :wxButton)'] = 1
+            self.dependencies.add( '(use-package :wxButton)' )
         else:
             stmt = ''
 
         return stmt
 
     def _get_class_filename(self, klass):
-        filename = os.path.join( self.out_dir, klass.replace('.', '_') + '.lisp' )
-        return filename
+        return os.path.join( self.out_dir, klass.replace('.', '_') + '.lisp' )
 
     def format_generic_access(self, obj):
-        if obj.is_toplevel:
+        if obj.IS_CLASS:
             return '(slot-top-window obj)'
-        else:
-            obj_name = self._format_name(obj.name)
-            if self.store_as_attr(obj):
-                return '(slot-%s obj)' % obj_name
-            else:
-                return obj_name
+        obj_name = self._format_name(obj.name)
+        if self.store_as_attr(obj):
+            return '(slot-%s obj)' % obj_name
+        return obj_name
 
     def _format_name(self, name):
         return name.replace('_', '-')
 
 
-writer = LispCodeWriter()  # The code writer is an instance of L{LispCodeWriter}.
+writer = LispCodeWriter()  # The code writer is an instance of LispCodeWriter.
 
 language = writer.language  # Language generated by this code generator
