@@ -3,52 +3,92 @@ Miscellaneous stuff, used in many parts of wxGlade
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2014-2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import common, config, compat
-import logging, os, re
+import logging, os, re, time
 import wx
 
 
-use_menu_icons = None
-
-# the SizerSlot which has the "mouse focus"; used to restore the mouse cursor if the user cancelled adding a widget
-currently_under_mouse = None
 
 _get_xpm_bitmap_re = re.compile(r'"(?:[^"]|\\")*"')
 _item_bitmaps = {}
 
-design_windows = []
+
+# handle currently focused widget ######################################################################################
 
 focused_widget = None  # the currently selected widget in GUI mode (for tree and property_panel)
+_next_focused_widget = None  # for delayed setting, to ensure that only the last call has an effect
+focused_time = 0.0  # sometimes, widgets ignore focus related events during a dead time of e.g. 50ms after setting focus
 
-def set_focused_widget(widget, force=False):
+flush_functions = []  # these will be called before the focused widget is set; currently only used by new_properties
+
+
+def set_focused_widget(widget, force=False, delayed=False):
     if not config.use_gui: return
+    global focused_widget, _next_focused_widget, focused_time
+
+    for f in flush_functions:
+        f()
+
+    if delayed:
+        _next_focused_widget = widget
+        wx.CallAfter(_set_focused_widget, widget, force)
+        return
+
+    _next_focused_widget = None  # cancel pending delayed call
+
     # set focused widget; tell tree and property panel
-    global focused_widget
     if focused_widget:
         focused_widget.update_view(selected=False)
+        set_focus = focused_widget.WX_CLASS in ("wxSpinCtrl", "wxSpinCtrlDouble")
+    else:
+        set_focus = False
     focused_widget = widget
     common.app_tree.set_current_widget(widget)
     common.property_panel.set_widget(widget, force)
+    if common.history: common.history.set_widget(widget)
+    common.main.set_widget(widget)  # to update menu and toolbar
+    if common.shell:
+        common.shell.txt_path.SetValue( widget and widget.get_path() or "" )
+
+    focused_time = time.time()
     if widget and widget.widget:
         # ensure that it is visible and selection is displayed, if applicable
         show_widget(widget)
         widget.update_view(selected=True)
+        # set focus in Design window to move away from certain widgets
+        if set_focus and hasattr(widget.widget, "HasFocus") and not widget.widget.HasFocus():
+            widget.widget.SetFocus()
+
+
+def _set_focused_widget(widget, force=False):
+    if not widget is _next_focused_widget:
+        # another call was made inbetween
+        return
+    set_focused_widget(widget, force)
+
+
+def rebuild_tree(widget=None, recursive=True, focus=True, freeze=False):
+    # re-build tree control for the widget and it's children; set focus to it; called after creation or modification
+    common.root.saved = False
+    common.app_tree.build(widget, recursive, freeze)
+    if focus and widget is not None:
+        set_focused_widget(widget, force=widget==common.root)
 
 
 def show_widget(widget):
     # ensure that notebook pages are selected such that widget is visible
     if not widget.widget: return
     while True:
-        if not widget.node or not widget.node.parent: break  # Application.node is None
-        parent = widget.node.parent.widget
+        if not widget.parent: break  # Application.node is None
+        parent = widget.parent
         if parent.__class__.__name__=="EditNotebook":
             # a widget under a wxNotebook without a panel: select page
-            if parent.widget and widget in parent.pages:
-                parent.widget.SetSelection( parent.pages.index(widget) )
+            if parent.widget and widget in parent.children:
+                parent.widget.SetSelection( parent.children.index(widget) )
 
         widget = parent  # go up one level
 
@@ -117,6 +157,7 @@ class SelectionMarker(object):
         self.owner = owner
         self.parent = parent
         if wx.Platform == '__WXMSW__': self.parent = owner
+        assert isinstance(self.parent, wx.Window)
         self.tag_pos = None
         self.tags = None
         self.update()
@@ -144,6 +185,7 @@ class SelectionMarker(object):
         if self.visible != visible:
             self.visible = visible
             if self.visible:
+                if not self.parent: return
                 if not self.tags:
                     self.tags = [SelectionTag(self.parent) for i in range(4)]
                 for i,pos in enumerate(self.tag_pos):
@@ -194,7 +236,7 @@ def string_to_color(color):
 
 def format_for_version(version):
     "Return the version information in a string; e.g. format_for_version((2, 8)) -> '2.8'"
-    # see: L{wxglade.codegen.BaseLangCodeWriter.for_version}
+    # see: wxglade.codegen.BaseLangCodeWriter.for_version
     return '%s.%s' % version
 
 def format_supported_by(version):
@@ -207,8 +249,12 @@ def format_supported_by(version):
 
 
 def get_toplevel_parent(obj):
-    if not isinstance(obj, wx.Window):
+    if isinstance(obj, wx.Sizer):
+        obj = obj.ContainingWindow
+    elif not isinstance(obj, wx.Window):
         obj = obj.widget
+    if hasattr(obj, "ContainingWindow"):
+        obj = obj.ContainingWindow
     while obj and not obj.IsTopLevel():
         obj = obj.GetParent()
     return obj
@@ -216,16 +262,15 @@ def get_toplevel_parent(obj):
 
 def get_toplevel_widget(widget):
     from edit_windows import EditBase, TopLevelBase
-    from edit_sizers import Sizer, SizerSlot
-    if isinstance(widget, Sizer):
-        widget = widget.window
+    from edit_sizers import SizerSlot
+    if widget.IS_SIZER: widget = widget.window
     assert isinstance(widget, (EditBase,SizerSlot)), _("EditBase or SizerBase object needed")
     while widget and not isinstance(widget, TopLevelBase):
         widget = widget.parent
     return widget
 
 
-def check_wx_version(major, minor=0, release=0, revision=0):
+def check_wx_version_at_least(major, minor=0, release=0, revision=0):
     "returns True if the current wxPython version is at least major.minor.release"
     return wx.VERSION[:-1] >= (major, minor, release, revision)
 
@@ -234,42 +279,48 @@ def check_wx_version(major, minor=0, release=0, revision=0):
 ########################################################################################################################
 # error/warning/info messages
 def error_message(msg, title="Error", display_traceback=False):
-    wx.MessageBox( _(msg), _(title), wx.OK | wx.CENTRE | wx.ICON_ERROR )
+    if config.use_gui:
+        wx.MessageBox( _(msg), _(title), wx.OK | wx.CENTRE | wx.ICON_ERROR )
+    else:
+        logging.error(msg)
 
 def info_message(msg, title="Information"):
-    wx.MessageBox( _(msg), _(title), wx.OK | wx.CENTRE | wx.ICON_INFORMATION )
+    if config.use_gui:
+        wx.MessageBox( _(msg), _(title), wx.OK | wx.CENTRE | wx.ICON_INFORMATION )
+    else:
+        logging.info(msg)
 
 def warning_message(msg, title="Warning"):
-    wx.MessageBox( _(msg), _(title), wx.OK | wx.CENTRE | wx.ICON_WARNING )
+    if config.use_gui:
+        wx.MessageBox( _(msg), _(title), wx.OK | wx.CENTRE | wx.ICON_WARNING )
+    else:
+        logging.warning(msg)
 
 
 ########################################################################################################################
 # menu helpers
 
-def append_menu_item(menu, id, text, xpm_file_or_artid=None): # XXX change: move id to the end of the argument list?
-    global use_menu_icons
-    if use_menu_icons is None:
-        use_menu_icons = config.preferences.use_menu_icons
-    item = wx.MenuItem(menu, id, text)
-    if wx.Platform == '__WXMSW__':
-        path = 'msw/'
-    else:
-        path = 'gtk/'
-    path = os.path.join(config.icons_path, path)
-    if use_menu_icons and xpm_file_or_artid is not None:
+def append_menu_item(menu, id, text, file_or_artid=None, **kwargs): # XXX change: move id to the end of the argument list?
+    if compat.IS_CLASSIC and "helpString" in kwargs:
+        kwargs["help"] = kwargs["helpString"]
+        del kwargs["helpString"]
+    item = wx.MenuItem(menu, id, text, **kwargs)
+    if file_or_artid is not None:
+        path = 'msw/'  if wx.Platform == '__WXMSW__'  else  'gtk/'
+        path = os.path.join(config.icons_path, path)
         bmp = None
-        if not xpm_file_or_artid.startswith(b'wxART_'):
+        if not isinstance(file_or_artid, bytes) or not file_or_artid.startswith(b'wxART_'):
             try:
-                bmp = _item_bitmaps[xpm_file_or_artid]
+                bmp = _item_bitmaps[file_or_artid]
             except KeyError:
-                f = os.path.join(path, xpm_file_or_artid)
+                f = os.path.join(path, file_or_artid)
                 if os.path.isfile(f):
-                    bmp = _item_bitmaps[xpm_file_or_artid] = wx.Bitmap(f, wx.BITMAP_TYPE_XPM)
+                    bmp = _item_bitmaps[file_or_artid] = wx.Bitmap(f)
                 else:
                     bmp = None
         else:
-            # xpm_file_or_artid is an id for wx.ArtProvider
-            bmp = wx.ArtProvider.GetBitmap( xpm_file_or_artid, wx.ART_MENU, (16, 16) )
+            # file_or_artid is an id for wx.ArtProvider
+            bmp = wx.ArtProvider.GetBitmap( file_or_artid, wx.ART_MENU, (16, 16) )
         if bmp is not None:
             try:
                 item.SetBitmap(bmp)
@@ -317,9 +368,11 @@ def restore_focus(func):
 def _can_remove():
     global focused_widget
     if focused_widget is None or not hasattr(focused_widget, "remove"): return False
-    if focused_widget.klass=="sizerslot" and hasattr(focused_widget, "sizer"):
-        sizer = focused_widget.sizer
-        if sizer.is_virtual() or focused_widget.sizer._IS_GRIDBAG:
+    if focused_widget.IS_SLOT:
+        # XXX change this later on, to remove e.g. notebook pages, but not when parent.CHILDREN is an int
+        if focused_widget.parent.CHILDREN == -1: return True
+        if focused_widget.parent.WX_CLASS in ("wxNotebook",): return True
+        if not focused_widget.parent.IS_SIZER or focused_widget.sizer._IS_GRIDBAG:
             wx.Bell()
             return False
     return True
@@ -337,14 +390,14 @@ def _remove():
 @restore_focus
 def _cut():
     global focused_widget
-    if not _can_remove() or focused_widget.klass=="sizerslot":
+    if not _can_remove():
         wx.Bell()
         return
     clipboard.cut(focused_widget)
     #focused_widget = None
 
 def _copy():
-    if focused_widget is None or focused_widget.klass=="sizerslot":
+    if focused_widget is None or focused_widget.IS_SLOT:
         wx.Bell()
         return
     clipboard.copy(focused_widget)
@@ -361,10 +414,12 @@ def _paste():
 def _insert():
     global focused_widget
     if not focused_widget: return
-    if not hasattr(focused_widget, "sizer") or not hasattr(focused_widget, "pos"): return
-    if focused_widget.sizer._IS_GRIDBAG: return
-    method = getattr(focused_widget.sizer, "insert_slot", None)
-    if method: method(focused_widget.pos)
+    parent = focused_widget.parent
+    if not parent or not parent.IS_SIZER: return
+    if parent._IS_GRIDBAG: return
+    #method = getattr(focused_widget.sizer, "insert_slot", None)
+    method = getattr(parent, "insert_slot", None)
+    if method: method(focused_widget.index)
 
 @restore_focus
 def _add():
@@ -376,6 +431,9 @@ def _add():
         method = getattr(focused_widget.sizer, "add_slot", None)
     if method: method()
 
+
+currently_under_mouse = None  # set when the mouse enters a sizer slot widget; see edit_base.Slot/SizeSlot
+
 def _cancel():
     if not common.adding_widget: return
     common.adding_widget = common.adding_sizer = False
@@ -384,7 +442,69 @@ def _cancel():
         currently_under_mouse.SetCursor(wx.STANDARD_CURSOR)
     common.app_tree.SetCursor(wx.STANDARD_CURSOR)
     compat.SetToolTip(common.app_tree, "")
-    common.palette.user_message("Canceled")
+    common.main.user_message("Canceled")
+
+@restore_focus
+def drop():
+    global focused_widget
+    method = None
+    if not focused_widget: return
+    if not focused_widget.check_drop_compatibility():
+        wx.Bell()
+        return
+    if focused_widget.IS_SLOT:
+        method = getattr(focused_widget, "on_drop_widget", None)
+    elif common.adding_sizer:
+        method = getattr(focused_widget, "drop_sizer", None)
+    #if not method:
+        #if not hasattr(focused_widget, "sizer"): return
+        #method = getattr(focused_widget.sizer, "add_slot", None)
+    if method: method(None, False)  # don't reset, but continue adding
+
+
+@restore_focus
+def navigate(up):
+    # move up or down in tree
+    focus = focused_widget
+    if not focus:
+        # get from Tree widget
+        item = common.app_tree.GetFocusedItem()
+        focus = common.app_tree._GetItemData(item)
+    if focus is None: return
+    siblings = [focus]  if focus.IS_ROOT else  focus.parent.get_all_children()
+    if not siblings: return
+    idx = siblings.index(focus)
+    if up:
+        if idx>0:
+            focus = siblings[idx-1]
+            children = focus.get_all_children()
+            if children: focus = children[-1]
+        else:
+            # no upper sibling -> go up
+            focus = focus.parent
+    else:
+        # down: look for children
+        children = focus.get_all_children()
+        if children:
+            # go to first child
+            focus = children[0]
+        else:
+            if idx+1<len(siblings):
+                # go to next sibling
+                focus = siblings[idx+1]
+            else:
+                # go up one or more levels
+                while True:
+                    if not focus.parent: return
+                    siblings = focus.parent.get_all_children()
+                    if siblings:
+                        idx = siblings.index(focus)
+                        if idx+1<len(siblings):
+                            focus = siblings[idx+1]
+                            break
+                    focus = focus.parent
+
+    if focus: set_focused_widget(focus)
 
 
 # accelerator tables to enable keyboard shortcuts for the popup menus of the various widgets (remove, cut, copy, paste)
@@ -396,52 +516,92 @@ accel_table_editors = {
     ("C", ord('V')):     (_paste,  ()),
     ("C", ord('I')):     (_insert, ()),
     ("C", ord('A')):     (_add,    ()),
+
+    ("", wx.WXK_UP):     (navigate,  (True, )),
+    ("", wx.WXK_DOWN):   (navigate,  (False,)),
+
+    ("",  wx.WXK_ESCAPE):(_cancel, ()),
+
+    ("", wx.WXK_RETURN): (drop,    ()),
+}
+
+if wx.Platform == "__WXMAC__":
+    # on Windows this one would go up in the hierarchy when in the Tree control
+    accel_table_editors["", wx.WXK_BACK] = (_remove, ())
+
+
+# for the palette window
+accel_table_editors_palette = {
     ("",  wx.WXK_ESCAPE):(_cancel, ())
 }
+
 # for all windows
 accel_table = {
     ("C", ord('Z')):     ((common, "history","undo"), "focused_widget"),
     ("C", ord('Y')):     ((common, "history","redo"), "focused_widget"),
     ("C", ord('R')):     ((common, "history","repeat"), "focused_widget"),
-    ("",  wx.WXK_F2):    ((common,"palette","show_tree"),            ()),
-    ("",  wx.WXK_F3):    ((common,"palette","show_props_window"),    ()),
-    ("",  wx.WXK_F4):    ((common,"palette","raise_all"),            ()),
-    ("",  wx.WXK_F5):    ((common,"palette","preview"),              ()),
-    ("",  wx.WXK_F6):    ((common,"palette","show_design_window"),   ()),
-    ("C", ord('S')):     ((common,"palette","save_app"),             ()),
-    ("C", ord('G')):     ((common,"app_tree","app","generate_code"), ()),
 
-    ("C", ord('N')):     ((common,"palette","new_app"),          ()), 
-    ("C", ord('O')):     ((common,"palette","open_app"),             ()),
-    ("C", ord('Q')):     ((common,"palette","Close"),                ()),
+    ("",  wx.WXK_F2):    ((common,"main","show_tree"),            ()),
+    ("",  wx.WXK_F3):    ((common,"main","show_props_window"),    ()),
+    ("",  wx.WXK_F4):    ((common,"main","show_palette"),         ()),
+    ("",  wx.WXK_F5):    ((common,"main","preview"),              ()),
+    ("",  wx.WXK_F6):    ((common,"main","show_design_window"),   ()),
+    ("",  wx.WXK_F7):    ((common,"main","create_shell_window"),  ()),
+
+    ("",  wx.WXK_F8):    ((common,"main","show_props_window"),    ("Common",)),
+    ("C", ord('M')):     ((common,"main","show_props_window"),    ("Common",)),
+
+    ("",  wx.WXK_F9):    ((common,"main","show_props_window"),    ("Layout",)),
+    ("C", ord('L')):     ((common,"main","show_props_window"),    ("Layout",)),
+
+    ("",  wx.WXK_F10):   ((common,"main","show_props_window"),    ("Widget",)),
+    ("C", ord('W')):     ((common,"main","show_props_window"),    ("Widget",)),
+
+    ("",  wx.WXK_F11):   ((common,"main","show_props_window"),    ("Events",)),
+    ("C", ord('E')):     ((common,"main","show_props_window"),    ("Events",)),
+
+    ("",  wx.WXK_F12):   ((common,"main","show_props_window"),    ("Code",)),
+    ("C", ord('D')):     ((common,"main","show_props_window"),    ("Code",)),  # -> 'O'? and use 'D' for Design window?
+
+    ("C", ord('P')):     ((common,"main","pin_design_window"),    ()),
+
+    ("A", ord('1')):     ((common,"main","switch_layout"),        (0,)),
+    ("A", ord('2')):     ((common,"main","switch_layout"),        (1,)),
+    ("A", ord('3')):     ((common,"main","switch_layout"),        (2,)),
+
+    ("C", ord('S')):     ((common,"main","save_app"),             ()),
+    ("C", ord('G')):     ((common,"root","generate_code"), ()),
+
+    ("C", ord('N')):     ((common,"main","new_app"),              ()), 
+    ("C", ord('O')):     ((common,"main","open_app"),             ()),
+    ("C", ord('Q')):     ((common,"main","Close"),                ()),
 }
 
-def handle_key_event(event, is_filter=False, toplevel_obj=None):
-    "centralized handler for Ctrl+C/X/V or Del key"
-    evt_key = event.GetKeyCode()
-    if toplevel_obj is None:
-        obj = event.GetEventObject()
-        if obj is not None:
-            toplevel_obj = obj.GetTopLevelParent()
 
-    if not is_filter:
-        if focused_widget and evt_key in (wx.WXK_UP, wx.WXK_DOWN) and toplevel_obj:
-            if toplevel_obj!=common.app_tree.GetParent() and toplevel_obj!=common.palette and toplevel_obj!=common.property_panel:
-                # must be a design window
-                navigate( evt_key==wx.WXK_UP )
-                return True
+def handle_key_event(event, window_type, window=None):
+    "centralized key event handler; called from EVT_CHAR_HOOK handlers"
+    # window_type: "design", "preview", "editor", "tree"    "properties", "palette" or None
+    evt_key = event.GetKeyCode()
+
+    tables = [accel_table]  # for all windows
+    if window_type in ("editor", "design", "tree"):
+        tables += [accel_table_editors]
+    elif window_type == "palette":
+        tables.append( accel_table_editors_palette )
 
     evt_flags = []
     if event.ControlDown(): evt_flags.append("C")
     if event.ShiftDown():   evt_flags.append("S")
+    if event.AltDown():     evt_flags.append("A")
     evt_flags = "".join(evt_flags)
 
-    handler = accel_table.get( (evt_flags,evt_key) )
-    if not handler and toplevel_obj:
-        if toplevel_obj in design_windows or toplevel_obj is common.app_tree.GetTopLevelParent():
-            handler = accel_table_editors.get( (evt_flags,evt_key) )
+    handler = None
+    for table in tables:
+        handler = table.get( (evt_flags,evt_key) )
+        if handler: break
     if not handler:
-        return False
+        event.Skip()
+        return
 
     function, args = handler
     if isinstance(function, tuple):
@@ -453,77 +613,9 @@ def handle_key_event(event, is_filter=False, toplevel_obj=None):
     if isinstance(args, str):
         args = (globals()[args],)
     wx.CallAfter(function, *args)
-    return True
-
-def on_key_down_event(event, toplevel_obj=None):
-    "centralized event handler for Ctrl+C/X/V or Del key; this will call event.Skip() for unhandled keys"
-    handled = handle_key_event(event, toplevel_obj=toplevel_obj)
-    if not handled: event.Skip()
-
-
-
-if config.use_gui and hasattr(wx, "EventFilter") and not config.testing:
-    # for filtering key down events
-    class PreviewEventFilter(wx.EventFilter):
-        windows = []
-        def add_window(self, window):
-            if not self.windows:
-                common.palette.AddFilter(self)
-            self.windows.append(window)
-        def remove_window(self, window):
-            self.windows.remove(window)
-            if not self.windows:
-                common.palette.RemoveFilter(self)
-        def FilterEvent(self, event):
-            t = event.GetEventType()
-            if t == wx.EVT_KEY_DOWN.typeId:
-                # handle the event only for Preview windows
-                toplevel = event.GetEventObject().GetTopLevelParent()
-                if toplevel in self.windows:
-                    handled = handle_key_event(event, is_filter=True)
-                    if handled:
-                        return self.Event_Processed
-            # Continue processing the event normally as well.
-            return self.Event_Skip
-    preview_event_filter = PreviewEventFilter()
-else:
-    preview_event_filter = None
-
-
-
-def navigate(up):
-    # must be a design window
-    focused_item = focused_widget.node.item
-    if up:
-        item = common.app_tree.GetPrevSibling(focused_item)
-        if not item:
-            # no upper sibling -> go up
-            item = common.app_tree.GetItemParent(focused_item)
-            #item = common.app_tree.GetFirstChild(focused_item)
-        else:
-            # go down again
-            while common.app_tree.ItemHasChildren(item):
-                item = common.app_tree.GetLastChild(item)
-    else:
-        if common.app_tree.ItemHasChildren(focused_item):
-            item, token = common.app_tree.GetFirstChild(focused_item)
-        else:
-            item = common.app_tree.GetNextSibling(focused_item)
-            if not item:
-                item = focused_item
-                while True:
-                    parent = common.app_tree.GetItemParent(item)
-                    if not parent: return
-                    if common.app_tree.ItemHasChildren(parent):
-                        item = common.app_tree.GetNextSibling(parent)
-                        if common.app_tree._GetItemData( common.app_tree.GetNextSibling(parent) ):
-                            break
-                    item = parent
-
-    widget = getattr(common.app_tree._GetItemData(item), "widget", None)
-    if not widget: return
-    set_focused_widget(widget)
 
+# for key handler in palette window:
+palette_hotkeys = {"S":"Sizers"}  # key to section name
 
 
 def _reverse_dict(src):
@@ -541,6 +633,26 @@ def dummy_contextmanager():
     yield
 
 
+if wx.Platform == '__WXMAC__':
+    # on Mac OS we need to disable STAY_ON_TOP when a dialog is to be shown
+    @contextlib.contextmanager
+    def disable_stay_on_top(widget):
+        # used like 'with parent and parent.frozen() or dummy_contextmanager()
+        restore = False
+        tl = get_toplevel_parent(widget)
+        if isinstance(tl, wx.Frame) and tl.GetWindowStyle() & wx.STAY_ON_TOP:
+            tl.ToggleWindowStyle(wx.STAY_ON_TOP)
+            restore = True
+        yield
+        if restore:
+            tl.ToggleWindowStyle(wx.STAY_ON_TOP)
+else:
+    # on other platforms, we don't need it
+    @contextlib.contextmanager
+    def disable_stay_on_top(widget):
+        yield
+    
+
 ########################################################################################################################
 # key handlers
 if wx.Platform == '__WXMAC__':
@@ -563,7 +675,7 @@ def wxstr(s, encoding=None):
     if encoding is None:
         if common.app_tree is None:
             return str(s)
-        encoding = common.app_tree.app.encoding
+        encoding = common.root.encoding
     if isinstance(s, compat.unicode): return s
     s = str(s)
     if isinstance(s, compat.unicode): return s
@@ -588,27 +700,49 @@ def get_xpm_bitmap(path):
                 try:
                     data = zipfile.ZipFile(archive).read(name)
                     data = [d[1:-1] for d in _get_xpm_bitmap_re.findall(data)]
-                    bmp = wx.BitmapFromXPMData(data)
+                    bmp = wx.BitmapFromXPMData(data)  # XXX
                 except:
                     logging.exception(_('Internal Error'))
                     bmp = wx.NullBitmap
     else:
-        bmp = wx.Bitmap(path, wx.BITMAP_TYPE_XPM)
+        bmp = wx.Bitmap(path)
     return bmp
 
 
 def get_absolute_path(path, for_preview=False):
     "Get an absolute path relative to the current output directory (where the code is generated)."
+    if os.path.sep!="\\":
+        path = path.replace("\\", os.path.sep)
     if os.path.isabs(path):
         return path
-    p = common.app_tree.app.output_path
+    p = common.root.output_path
     if for_preview:
-        p = getattr(common.app_tree.app, 'real_output_path', u'')
+        p = getattr(common.root, 'real_output_path', u'')
+    if os.path.sep!="\\":
+        p = p.replace("\\", os.path.sep)
     if not os.path.isabs(p):
         # a path relative to the application filename
-        appdir = os.path.dirname(common.app_tree.app.filename)
+        appdir = os.path.dirname(common.root.filename)
         p = os.path.abspath( os.path.join(appdir, p) )
 
     if not os.path.isdir(p): p = os.path.dirname(p)
 
     return os.path.abspath( os.path.join(p, path) )
+
+
+def get_relative_path(filename):
+    # returns a relative path if filename is inside the project directory
+    if filename:
+        project_path = common.root.filename
+        if project_path:
+            project_path = os.path.abspath( os.path.dirname(project_path) )
+            if filename.startswith(project_path):
+                filename = "./" + os.path.relpath(filename, project_path)
+        if os.path.sep=="\\":
+            filename = filename.replace("\\", "/")
+    return filename
+
+
+def RelativeFileSelector(message="Select the file", *args, **kwargs):
+    # returns a relative path if the selected file is inside the project directory
+    return get_relative_path( wx.FileSelector(_(message), *args, **kwargs) )
