@@ -2,7 +2,7 @@
 Graphical tests
 
 @copyright: 2012-2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
@@ -54,10 +54,10 @@ class TestGui(WXGladeGUITest):
         self.load_and_generate('ComplexExample_30', excluded=["lisp"])
         #self.load_and_generate('ComplexExample_30', included=["lisp"])
         # Lisp code has to raise an exception
-        common.app_tree.app.properties["language"].set("lisp")
+        common.app_tree.root.properties["language"].set("lisp")
         self._process_wx_events()
-        common.app_tree.root.widget.generate_code()
-        self._assert_error_message("Generating Lisp code for wxWidgets version 3.0 is not supported")
+        common.app_tree.root.generate_code()
+        self._assert_error_message("Generating Lisp code")
 
     def test_CodeGeneration_CustomWidget(self):
         'Test GUI code generation using "CustomWidget"'
@@ -86,6 +86,10 @@ class TestGui(WXGladeGUITest):
         self.load_and_generate('FontTest28', test_GUI=True)
         self.load_and_generate('FontTest', test_GUI=False)
 
+    def test_Frame_Size(self):
+        "Generate Fit() code for the second frame"
+        self.load_and_generate('Frame_Size', test_GUI=False)
+
     def test_Grid(self):
         "Test code generation with a grid widgets and handling events"
         self.load_and_generate('Grid', test_GUI=False)
@@ -112,19 +116,19 @@ class TestGui(WXGladeGUITest):
         config.preferences.default_border = False
         config.preferences.default_border_size = 0
 
-        self.load_and_generate('Python_Preferences', included=["Python"], test_GUI=True)
-        self.load_and_generate('Perl_Preferences', included=["Perl"], test_GUI=False, preview=False)
+        self.load_and_generate('Python_Preferences', included=["python"], test_GUI=True)
+        self.load_and_generate('Perl_Preferences', included=["perl"], test_GUI=False, preview=False)
         self.load_and_generate('CPP_Preferences', included=["C++"], test_GUI=True)
-        self.load_and_generate('Lisp_Preferences', included=["Lisp"], test_GUI=True)
+        self.load_and_generate('Lisp_Preferences', included=["lisp"], test_GUI=True)
 
         # with default border
         config.preferences.default_border = True
         config.preferences.default_border_size = 5
 
-        self.load_and_generate('Python_Preferences', included=["Python"], test_GUI=True)
-        self.load_and_generate('Perl_Preferences', included=["Perl"], test_GUI=False, preview=False)
+        self.load_and_generate('Python_Preferences', included=["python"], test_GUI=True)
+        self.load_and_generate('Perl_Preferences', included=["perl"], test_GUI=False, preview=False)
         self.load_and_generate('CPP_Preferences', included=["C++"], test_GUI=True)
-        self.load_and_generate('Lisp_Preferences', included=["Lisp"], test_GUI=True)
+        self.load_and_generate('Lisp_Preferences', included=["lisp"], test_GUI=True)
 
         config.preferences.default_border, config.preferences.default_border_size = restore
 
@@ -135,6 +139,30 @@ class TestGui(WXGladeGUITest):
         # store sizer references
         self.load_and_generate('Sizers_classattr', test_GUI=False)
 
+    def test_bases_etc(self):
+        # test changes in data structure and code generation from 0.9 to 1.0
+        self.load_and_generate('BasesEtc_w_sizers', test_GUI=True)  # test files generated with v0.9.5 and re-ordered
+        self.load_and_generate('BasesEtc', test_GUI=True)           # test files generated with 0.9.9pre
+
+    def test_grid_custom_base(self):
+        # test custom base for grid and also "Mark code block" deactivated
+        self.load_and_generate('test_grid_custom_base', test_GUI=True)
+
+    def test_keep_code_migration(self):
+        # test migration from 0.9 to 1.0: __do_layout and __set_properties should be removed
+        # C++: "virtual" will not be added here:  "public: void on_button_plot(wxCommandEvent &event);"
+        # copy old files
+        for ext in ("py","cpp","pl","lisp", "h"):
+            old_filename = self._get_casefile_path('matplotlib_example_old.%s'%ext)
+            generate_filename = self._get_outputfile_path('matplotlib_example.%s'%ext)
+            self._copy_and_modify(old_filename, generate_filename)
+        # the standard test will do the rest
+        self.load_and_generate('matplotlib_example', test_GUI=True)
+
+    def test_class_migration(self):
+        # test migration from 0.9 to 1.0
+        self.load_and_generate('test_no_custom_class09', test_GUI=True)
+
     def test_Python_Ogg1(self):
         "Test Python code generation with overwriting a single existing file, preserving manually added code"
         # set up filenames, copy the old file to the output path and modify it to trigger re-writing
@@ -145,12 +173,12 @@ class TestGui(WXGladeGUITest):
 
         # load and set up project
         common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-        common.app_tree.app.properties["overwrite"].set(False)  # overwrite is 0 in the file
-        common.app_tree.app.properties["output_path"].set(generate_filename)
-        common.app_tree.app.properties["language"].set("python")
+        common.app_tree.root.properties["overwrite"].set(False)  # overwrite is 0 in the file
+        common.app_tree.root.properties["output_path"].set(generate_filename)
+        common.app_tree.root.properties["language"].set("python")
         # generate, compare and check for overwriting
         self._process_wx_events()
-        common.app_tree.root.widget.generate_code()
+        common.app_tree.root.generate_code()
         self._compare_files(expected_filename, generate_filename, check_mtime=True)
 
     def test_Python_Ogg2(self):
@@ -170,12 +198,12 @@ class TestGui(WXGladeGUITest):
 
         # load and set up project
         common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-        common.app_tree.app.properties["overwrite"].set(False)  # overwrite is 0 in the file
-        common.app_tree.app.properties["output_path"].set(self.outDirectory)
-        common.app_tree.app.properties["language"].set("python")
+        common.app_tree.root.properties["overwrite"].set(False)  # overwrite is 0 in the file
+        common.app_tree.root.properties["output_path"].set(self.outDirectory)
+        common.app_tree.root.properties["language"].set("python")
         # generate, compare and check for overwriting
         self._process_wx_events()
-        common.app_tree.root.widget.generate_code()
+        common.app_tree.root.generate_code()
         self._compare_files(expected_app,    generate_app, check_mtime=True)
         self._compare_files(expected_dialog, generate_dialog, check_mtime=True)
         self._compare_files(expected_frame,  generate_frame, check_mtime=True)
@@ -197,13 +225,15 @@ class TestGui(WXGladeGUITest):
     
             # load and set up project
             common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-            common.app_tree.app.properties["overwrite"].set(False)  # overwrite is 0 in the file already
-            common.app_tree.app.properties["output_path"].set(generate_filename)
-            common.app_tree.app.properties["language"].set(language)
+            common.app_tree.root.properties["overwrite"].set(False)  # overwrite is 0 in the file already
+            common.app_tree.root.properties["output_path"].set(generate_filename)
+            common.app_tree.root.properties["language"].set(language)
             # generate, compare and check for overwriting
             self._process_wx_events()
-            common.app_tree.root.widget.generate_code()
+            common.app_tree.root.generate_code()
             self._compare_files(expected_filename, generate_filename, check_mtime=True)
+            if language=="C++":
+                self._compare_files(expected_filename_h, generate_filename_h, check_mtime=True)
 
     def test_all_Ogg2(self):
         "Test Python code generation with overwriting a single existing file, preserving manually added code"
@@ -221,13 +251,15 @@ class TestGui(WXGladeGUITest):
     
             # load and set up project
             common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-            common.app_tree.app.properties["overwrite"].set(False)  # overwrite is 0 in the file
-            common.app_tree.app.properties["output_path"].set(generate_filename)
-            common.app_tree.app.properties["language"].set(language)
+            common.app_tree.root.properties["overwrite"].set(False)  # overwrite is 0 in the file
+            common.app_tree.root.properties["output_path"].set(generate_filename)
+            common.app_tree.root.properties["language"].set(language)
             # generate, compare and check for overwriting
             self._process_wx_events()
-            common.app_tree.root.widget.generate_code()
+            common.app_tree.root.generate_code()
             self._compare_files(expected_filename, generate_filename, check_mtime=True)
+            if language=="C++":
+                self._compare_files(expected_filename_h, generate_filename_h, check_mtime=True)
 
     def test_all_Ogg2(self):
         "Test code generation with overwriting multiples existing files, preserving manually added code"
@@ -259,53 +291,60 @@ class TestGui(WXGladeGUITest):
 
             # load and set up project
             common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-            common.app_tree.app.properties["overwrite"].set(False)  # overwrite is 0 in the file
-            common.app_tree.app.properties["output_path"].set(self.outDirectory)
-            common.app_tree.app.properties["language"].set(language)
+            common.app_tree.root.properties["overwrite"].set(False)  # overwrite is 0 in the file
+            common.app_tree.root.properties["output_path"].set(self.outDirectory)
+            common.app_tree.root.properties["language"].set(language)
             # generate, compare and check for overwriting
             self._process_wx_events()
             if language=="C++":
-                common.app_tree.app.app_filename = P+APP
-            common.app_tree.root.widget.generate_code()
+                common.app_tree.root.app_filename = P+APP
+            common.app_tree.root.generate_code()
             check_mtime = language!="perl"
             self._compare_files(expected_app,    generate_app,    check_mtime=check_mtime)
             self._compare_files(expected_dialog, generate_dialog, check_mtime=check_mtime)
             self._compare_files(expected_frame,  generate_frame,  check_mtime=check_mtime)
 
+            if language=="C++":
+                self._compare_files(expected_filename_dialog_h, generate_filename_dialog_h, check_mtime=True)
+                self._compare_files(expected_filename_frame_h, generate_filename_frame_h, check_mtime=True)
+
+    def test_all_Tool_Menu_EventBinding(self):
+        self.load_and_generate('Tool_Menu_EventBinding', excluded=["lisp"], test_GUI=False)
+
     def test_OutputFileAndDirectory(self):
         "Test check for output file and directory"
         infilename = self._get_casefile_path('Python_Preferences.wxg')
         common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
 
         # Single output file out_path shouldn't be a directory, non-existing file or non-writable directory
-        common.app_tree.app.properties["output_path"].set(self.outDirectory)
-        common.app_tree.root.widget.generate_code()
-        self._assert_error_message( "can not be a directory when generating a single file" )
+        common.app_tree.root.properties["output_path"].set(self.outDirectory)
+        common.app_tree.root.generate_code()
+        self._assert_error_message( "path is directory" )
 
-        common.app_tree.app.properties["output_path"].set( os.path.join(self.outDirectory,"non-existing/result.py") )
-        common.app_tree.root.widget.generate_code()
-        self._assert_error_message( "must be an existing directory" )
+        common.app_tree.root.properties["output_path"].set( os.path.join(self.outDirectory,"non-existing/result.py") )
+        common.app_tree.root.generate_code()
+        self._assert_error_message( "directory does not exist" )
 
-        #common.app_tree.app.properties["output_path"].set( os.path.join(self.outDirectory,"non-writable/result.py") )
-        #common.app_tree.root.widget.generate_code()
+        #common.app_tree.root.properties["output_path"].set( os.path.join(self.outDirectory,"non-writable/result.py") )
+        #common.app_tree.root.generate_code()
         #self._assert_error_message( "" )
 
         # Multiple output file out_path should be a writable directory
-        common.app_tree.app.properties["multiple_files"].set(1)
-        common.app_tree.app.properties["output_path"].set( os.path.join(self.outDirectory,"non-existing") )
-        common.app_tree.root.widget.generate_code()
-        self._assert_error_message( " must be an existing directory" )
+        common.app_tree.root.properties["multiple_files"].set(1)
+        common.app_tree.root.properties["output_path"].set( os.path.join(self.outDirectory,"non-existing") )
+        common.app_tree.root.generate_code()
+        self._assert_error_message( "directory does not exist" )
 
-        #common.app_tree.app.properties["output_path"].set( os.path.join(self.outDirectory,"non-writable") )
-        #common.app_tree.root.widget.generate_code()
+        #common.app_tree.root.properties["output_path"].set( os.path.join(self.outDirectory,"non-writable") )
+        #common.app_tree.root.generate_code()
         #self._assert_error_message( "can not be a directory when generating a single file" )
 
     def test_PythonSubclass(self):
         "check for correct import: from package.name import name"
         infilename = self._get_casefile_path('PythonSubclass.wxg')
         common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-        common.app_tree.app.filename = self._get_outputfile_path('PythonSubclass.wxg')
-        common.app_tree.root.widget.generate_code()
+        common.app_tree.root.filename = self._get_outputfile_path('PythonSubclass.wxg')
+        common.app_tree.root.generate_code()
 
         expected_filename = self._get_casefile_path("PythonSubclass.py")
         generated_filename = self._get_outputfile_path("PythonSubclass.py")
@@ -318,17 +357,17 @@ class TestGui(WXGladeGUITest):
         "Test for multi file XRC projects."
         infilename = self._get_casefile_path('Python_Preferences.wxg')
         common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-        common.app_tree.app.properties["multiple_files"].set(1)
-        common.app_tree.app.properties["language"].set("XRC")
-        common.app_tree.root.widget.generate_code()
+        common.app_tree.root.properties["multiple_files"].set(1)
+        common.app_tree.root.properties["language"].set("XRC")
+        common.app_tree.root.generate_code()
         self._assert_error_message( "XRC code cannot be split into multiple files" )
 
     def test_WxgTemplateCodegenNotPossible(self):
         "Test for code generation from a template"
         infilename = self._get_casefile_path('Python_Preferences.wxg')
         common.main._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-        common.app_tree.app.properties["is_template"].set(True)
-        common.app_tree.root.widget.generate_code()
+        common.app_tree.root.properties["is_template"].set(True)
+        common.app_tree.root.generate_code()
         self._assert_error_message( "Code generation from a template is not possible" )
 
     def test_SizersSize(self):
@@ -341,6 +380,14 @@ class TestGui(WXGladeGUITest):
     def test_Menu(self):
         self.load_and_generate('MenuTest', excluded=["lisp"], test_GUI=True)
 
+    def test_Menu_lambda_handlers_keep_code(self):
+        # copy .py file to test "Keep user code"
+        old_filename = self._get_casefile_path('MenuHandlers_Lambda.py')
+        generate_filename = self._get_outputfile_path("MenuHandlers_Lambda.py")
+        self._copy_and_modify(old_filename, generate_filename)
+        # the standard test will do the rest
+        self.load_and_generate('MenuHandlers_Lambda', included=["python"], test_GUI=True)
+
     def _assert_styles(self, got, expected, msg=None):
         if isinstance(got,      str): got      = got.split("|")
         if isinstance(expected, str): expected = expected.split("|")
@@ -354,28 +401,13 @@ class TestGui(WXGladeGUITest):
         # XXX actually generate a file and simulate editing
 
         #common.palette._open_app(infilename, use_progress_dialog=False, add_to_history=False)
-        common.app_tree.clear()
-        common.app_tree.app.init()
+        common.app_tree.root.clear()
+        common.app_tree.root.init()
         import widgets.frame.frame
-        widgets.frame.frame.builder(None,None,0, "wxFrame", "MyFrame", "frame")
+        frame = widgets.frame.frame.builder(common.root, 0, "wxFrame", "MyFrame", "frame")
 
-        item = common.app_tree.root.children[0]
-        common.adding_widget = True
-        common.widget_to_add = "EditHyperlinkCtrl"
-        item.widget.sizer.children[1].on_drop_widget(None)
-        hyperlink = item.widget.sizer.children[1]
-
-        ## expand tree and show edit window
-        #tree = common.app_tree.drop_target()
-        #root = tree.GetRootItem()
-        #first, cookie = tree.GetFirstChild(root)
-        #if first.IsOk():
-            #tree.expand()
-            #self._process_wx_events()
-            #tree.SelectItem(first)
-            #self._process_wx_events()
-            #node = tree.GetPyData(first)
-            #tree.show_toplevel(node)
+        import widgets.hyperlink_ctrl.hyperlink_ctrl
+        hyperlink = widgets.hyperlink_ctrl.hyperlink_ctrl.builder(frame, 0, "Hyperlink")
         self._process_wx_events()
         # check available style names
         sp = hyperlink.properties["style"]
@@ -456,16 +488,13 @@ class TestGui(WXGladeGUITest):
     def test_missing_application_attributes(self):
         #"Test load wxg file w/ missing <application> attributes and generate code"
         "convert .xrc file with missing <application> attributes to .wxg and load it"
+        # this will fail on Python 3.8 as the ordering of elements is not defined
         # convert .xrc to .wxg
         infilename  = self._get_casefile_path('app_wo_attrs_gui.xrc')
         generated_filename = self._get_outputfile_path('app_wo_attrs_gui.wxg')
         xrc2wxg.convert(infilename, generated_filename)
         # compare
-        if sys.version_info[:2]< (3,8):
-            # before 3.8 attribute names were ordered, now they are preserved
-            expected_filename = self._get_casefile_path('app_wo_attrs_gui.wxg')
-        else:
-            expected_filename = self._get_casefile_path('app_wo_attrs_gui_new.wxg')
+        expected_filename = self._get_casefile_path('app_wo_attrs_gui.wxg')
         self._compare_files(expected_filename, generated_filename)
         # open the .wxg file; there should be no problem
         self._messageBox = None
@@ -474,12 +503,21 @@ class TestGui(WXGladeGUITest):
 
     def test_toplevels_no_size(self):
         "Test frame, panel, dialog without size"
+        # previous versions wrote only the last of the three panels
         self.load_and_generate('toplevels_no_size', test_GUI=True)
 
+    def test_toplevels_code_pre_post(self):
+        "Test frame, dialog without extracode_pre, extracode_post"
+        self.load_and_generate('toplevel_extracode', test_GUI=False)
+
     def test_unsupported_flags(self):
         "Test code generation with unsupported flags"
         self.load_and_generate('no_supported_flags', test_GUI=False)
 
+    def test_panel_class(self):
+        "Test code generation for a standalone panel class"
+        self.load_and_generate('PanelClass', excluded=("lisp",), test_GUI=False)
+
     @unittest.skipIf(wx.VERSION[:2]<(3,0), "not all supported (import_test.xrc fails")
     def test_load_xrc(self):
         "Test loading XRC files"
