@@ -3,7 +3,7 @@ Code generator functions for CustomWidget objects
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2016 Carsten Grohmann
-@copyright: 2017 Dietmar Schwertberger
+@copyright: 2017-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
@@ -34,27 +34,23 @@ def format_ctor_arguments(arguments, parent, id, size):
 class PythonCustomWidgetGenerator(wcodegen.PythonWidgetCodeWriter):
     def get_code(self, widget):
         #if self.codegen.preview and widget.klass not in widget.parser.class_names:
-        if self.codegen.preview and widget.klass not in self.codegen.class_names:
+        if self.codegen.preview:
             # if this CustomWidget refers to another class in the same wxg
             # file, use that for the preview
             return self.get_code_preview(widget)
         prop = widget.properties
         id_name, id = self.codegen.generate_code_id(widget)
-        parent = self.format_widget_access(widget.parent)
+        parent = self.format_widget_access(widget.parent_window)
         init = []
         if id_name: init.append(id_name)
         arguments = format_ctor_arguments( widget.arguments, parent, id, widget.size)
-        cust_ctor = widget.custom_ctor.strip()
-        if cust_ctor:
-            ctor = cust_ctor
-        else:
-            ctor = widget.klass
+        ctor = widget.custom_ctor.strip() or widget.instance_class
         init.append( 'self.%s = %s(%s)\n' % (widget.name, ctor, ", ".join(arguments)) )
-        props_buf = self.codegen.generate_common_properties(widget)
-        return init, props_buf, []
+        init += self.codegen.generate_code_common_properties(widget)
+        return init, []
 
     def get_code_preview(self, widget):
-        parent = self.format_widget_access(widget.parent)
+        parent = self.format_widget_access(widget.parent_window)
         init = []
         append = init.append
         append('self.%s = wx.Window(%s, -1, style=wx.FULL_REPAINT_ON_RESIZE)\n' % (widget.name, parent))
@@ -86,7 +82,7 @@ def self_%s_on_paint(event):
             append(line + '\n')
         append( 'self.%s.Bind(wx.EVT_PAINT, self_%s_on_paint)\n' % (widget.name, widget.name) )
         append( 'self.%s.Bind(wx.EVT_ERASE_BACKGROUND, lambda event: None)\n' % widget.name )
-        return init, [], []
+        return init, []
 
 
 class CppCustomWidgetGenerator(wcodegen.CppWidgetCodeWriter):
@@ -96,19 +92,14 @@ class CppCustomWidgetGenerator(wcodegen.CppWidgetCodeWriter):
             ids = [id_name]
         else:
             ids = []
-        if not widget.parent.is_toplevel:
-            parent = '%s' % widget.parent.name
-        else:
-            parent = 'this'
+
+        parent = self.format_widget_access(widget.parent_window)
+
         arguments = format_ctor_arguments( widget.arguments, parent, id, widget.size )
-        cust_ctor = widget.custom_ctor.strip()
-        if cust_ctor:
-            ctor = cust_ctor
-        else:
-            ctor = 'new ' + widget.klass
+        ctor = widget.custom_ctor.strip() or ('new ' + widget.instance_class)
         init = [ '%s = %s(%s);\n' % (widget.name, ctor, ", ".join(arguments)) ]
-        props_buf = self.codegen.generate_common_properties(widget)
-        return init, ids, props_buf, []
+        init += self.codegen.generate_code_common_properties(widget)
+        return init, ids, []
 
 
 
@@ -120,7 +111,7 @@ def xrc_code_generator(obj):
         def write(self, outfile, ntabs, properties=None):
             if properties is None: properties = {}
             # first, fix the class:
-            self.klass = obj.klass
+            self.subclass = self.klass = obj.instance_class
             # delete the custom constructor property
             properties['custom_ctor'] = None
             # then, the attributes:
