@@ -75,10 +75,10 @@ class CppDialogGenerator(wcodegen.CppWidgetCodeWriter):
         # SetAffirmativeId and SetEscapeId, if defined
         if obj.check_prop_truth("affirmative"):
             buttons = obj.find_children(obj.affirmative, "wxButton")
-            if buttons: ret.append( "SetAffirmativeId(%s->GetId())\n" % self.format_widget_access(buttons[0]) )
+            if buttons: ret.append( "SetAffirmativeId(%s->GetId());\n" % self.format_widget_access(buttons[0]) )
         if obj.check_prop_truth("escape"):
             buttons = obj.find_children(obj.escape, "wxButton")
-            if buttons: ret.append( "SetEscapeId(%s->GetId())\n" % self.format_widget_access(buttons[0]) )
+            if buttons: ret.append( "SetEscapeId(%s->GetId());\n" % self.format_widget_access(buttons[0]) )
         if ret: ret.append("\n")
 
         ret.append( 'Layout();\n' )
