@@ -15,22 +15,29 @@ class PythonGaugeGenerator(wcodegen.PythonWidgetCodeWriter):
 
     def _prepare_tmpl_content(self, obj):
         wcodegen.PythonWidgetCodeWriter._prepare_tmpl_content(self, obj)
-        self.tmpl_dict['range'] = obj.properties.get('range', '10')
+        self.tmpl_dict['range'] = obj.range
         return
 
-# end of class PythonGaugeGenerator
-
 
 class CppGaugeGenerator(wcodegen.CppWidgetCodeWriter):
-    tmpl = '%(name)s = new %(klass)s(%(parent)s, %(id)s, ' \
-           '%(range)s%(style)s);\n'
+    tmpl = '%(name)s = new %(klass)s(%(parent)s, %(id)s, %(range)s%(style)s);\n'
 
     def _prepare_tmpl_content(self, obj):
         wcodegen.CppWidgetCodeWriter._prepare_tmpl_content(self, obj)
-        self.tmpl_dict['range'] = obj.properties.get('range', '10')
+        self.tmpl_dict['range'] = obj.range
         return
 
-# end of class CppGaugeGenerator
+
+def xrc_code_generator(obj):
+    xrcgen = common.code_writers['XRC']
+
+    class XrcCodeGenerator(xrcgen.DefaultXrcObject):
+        def write(self, out_file, ntabs):
+            style = self.widget.properties["style"].get_string_value()
+            properties = {"style":style}
+            xrcgen.DefaultXrcObject.write(self, out_file, ntabs, properties)
+
+    return XrcCodeGenerator(obj)
 
 
 def initialize():
@@ -38,3 +45,4 @@ def initialize():
     common.class_names['EditGauge'] = klass
     common.register('python', klass, PythonGaugeGenerator(klass))
     common.register('C++', klass, CppGaugeGenerator(klass))
+    common.register('XRC',    klass, xrc_code_generator)
