@@ -9,7 +9,7 @@ Perl generator functions for wxMenuBar objects
 import common
 import wcodegen
 from MenuTree import *
-from codegen import MenuHandler
+#from .codegen import MenuHandler
 
 
 class PerlMenubarGenerator(wcodegen.PerlWidgetCodeWriter):
@@ -19,7 +19,7 @@ class PerlMenubarGenerator(wcodegen.PerlWidgetCodeWriter):
     def get_init_code(self, obj):
         out = []
         append = out.append
-        menus = obj.properties['menubar']
+        menus = obj.menus
         ids = []
         # We need to keep track of tmpnames used.
         tmpsused = {}
@@ -36,20 +36,21 @@ class PerlMenubarGenerator(wcodegen.PerlWidgetCodeWriter):
                     if name: ids.append(name)
                     id = val
 
+                label = self.codegen.quote_str(item.label)
+                help_str = self.codegen.quote_str(item.help_str)
+
                 if item.children:
                     if item.name:
                         name = item.name
                     else:
                         name = '%s_sub' % menu
-                        if not tmpsused.has_key(name):
+                        if not name in tmpsused:
                             tmpsused[name] = 1
                             append('my %s;\n' % name)
 
                     append('%s = Wx::Menu->new();\n' % name)
                     append_items(name, item.children)
-                    append('%s->Append(%s, %s, %s, %s);\n' %
-                           (menu, id, self.codegen.quote_str(item.label),
-                            name, self.codegen.quote_str(item.help_str)))
+                    append( '%s->Append(%s, %s, %s, %s);\n' % (menu, id, label, name, help_str) )
                 else:
                     item_type = 0
                     if item.checkable == '1':
@@ -63,17 +64,9 @@ class PerlMenubarGenerator(wcodegen.PerlWidgetCodeWriter):
                         itemname = ''
 
                     if item_type:
-                        append('%s%s->Append(%s, %s, %s, %s);\n' %
-                               (itemname, menu, id,
-                                self.codegen.quote_str(item.label),
-                                self.codegen.quote_str(item.help_str),
-                                item_type))
+                        append( '%s%s->Append(%s, %s, %s, %s);\n' % (itemname, menu, id, label, help_str, item_type) )
                     else:
-
-                        append('%s%s->Append(%s, %s, %s);\n' %
-                               (itemname, menu, id,
-                                self.codegen.quote_str(item.label),
-                                self.codegen.quote_str(item.help_str)))
+                        append( '%s%s->Append(%s, %s, %s);\n' % (itemname, menu, id, label, help_str) )
         #self._logger.debug('menus = %s', menus)
 
         obj_name = self.format_widget_access(obj)
@@ -94,48 +87,32 @@ class PerlMenubarGenerator(wcodegen.PerlWidgetCodeWriter):
         return ids + out
 
     def get_code(self, obj):
-        klass = obj.base
-        if klass != obj.klass:
-            klass = obj.klass
-        else:
-            klass = self.cn(klass)
+        klass = obj.get_instantiation_class(self.cn, self.cn_class)
 
-        init = ['\n\n', '# Menu Bar\n\n', '$self->{%s} = %s->new();\n' %
-                (obj.name, klass)]
+        init = [ '\n\n', '# Menu Bar\n\n', '$self->{%s} = %s->new();\n' % (obj.name, klass) ]
         init.extend(self.get_init_code(obj))
         init.append('$self->SetMenuBar($self->{%s});\n' % obj.name)
         init.append('\n')
         init.append('# Menu Bar end\n\n')
-        return init, [], []
+        return init, []
 
     def get_event_handlers(self, obj):
-        out = []
+        ret = []
 
         def do_get(item):
-            ret = []
-            if item.name:
-                val = self.codegen.add_object_format_name(item.name)
-            else:
-                name, val = self.codegen.generate_code_id(None, item.id)
-                if not val:
-                    val = '-1'  # but this is wrong anyway...
             if item.handler:
-                ret.append((val, 'EVT_MENU', item.handler, 'wxCommandEvent'))
-            if item.children:
-                for c in item.children:
-                    ret.extend(do_get(c))
-            return ret
+                ret.append((item, 'EVT_MENU', item.handler, 'wxCommandEvent'))
+            for c in item.children:
+                do_get(c)
 
-        for menu in obj.properties['menubar']:
-            out.extend(do_get(menu.root))
-        return out
+        for menu in obj.menus:
+            do_get(menu.root)
+
+        return ret
 
-# end of class PerlMenubarGenerator
 
 
 def initialize():
     klass = 'wxMenuBar'
     common.class_names['EditMenuBar'] = klass
-    common.toplevels['EditMenuBar'] = 1
-    common.register('perl', klass, PerlMenubarGenerator(klass),
-                    'menus', MenuHandler)
+    common.register('perl', klass, PerlMenubarGenerator(klass), ) #'menus', MenuHandler)
