@@ -8,16 +8,16 @@ wxStaticBitmap objects
 
 import wx
 
-import common, config
+import common, misc
 from edit_windows import ManagedBase, EditStylesMixin
 from gui_mixins import BitmapMixin
-from tree import Node
 import new_properties as np
 
 
-class EditStaticBitmap(ManagedBase, EditStylesMixin, BitmapMixin):
+class EditStaticBitmap(BitmapMixin, ManagedBase, EditStylesMixin):
     "Class to handle wxStaticBitmap objects"
-    update_widget_style = False
+    recreate_on_style_change = True
+    WX_CLASS = 'wxStaticBitmap'
     _PROPERTIES = ["Widget", "bitmap", "attribute", "style"]
     PROPERTIES = ManagedBase.PROPERTIES + _PROPERTIES + ManagedBase.EXTRA_PROPERTIES
     ManagedBase.MOVE_PROPERTY(PROPERTIES, "attribute", "name")
@@ -25,18 +25,17 @@ class EditStaticBitmap(ManagedBase, EditStylesMixin, BitmapMixin):
     _PROPERTY_HELP = {"attribute":'Store instance as attribute of window class; e.g. self.bitmap_1 = wx.wxStaticBitmap'
                                   '(...)\nWithout this, you can not access the bitmap from your program.'}
 
-    def __init__(self, name, parent, id, bmp_file, sizer, pos):
-        ManagedBase.__init__(self, name, 'wxStaticBitmap', parent, id, sizer, pos)
+    def __init__(self, name, parent, index, bmp_file):
+        ManagedBase.__init__(self, name, parent, index)
         EditStylesMixin.__init__(self)
 
         # initialise instance properties
-        filedialog_style = wx.FD_OPEN | wx.FD_FILE_MUST_EXIST  # for the following two properties
         self.bitmap    = np.BitmapProperty(bmp_file)
         self.attribute = np.CheckBoxProperty(False, default_value=False)
 
     def create_widget(self):
         bmp = self.get_preview_obj_bitmap()
-        self.widget = wx.StaticBitmap(self.parent.widget, self.id, bmp)
+        self.widget = wx.StaticBitmap(self.parent_window.widget, wx.ID_ANY, bmp, style=self.style)
         if wx.Platform == '__WXMSW__':
             def get_best_size():
                 bmp = self.widget.GetBitmap()
@@ -45,58 +44,40 @@ class EditStaticBitmap(ManagedBase, EditStylesMixin, BitmapMixin):
                 return wx.StaticBitmap.GetBestSize(self.widget)
             self.widget.GetBestSize = get_best_size
 
-    def properties_changed(self, modified=None):
+    def _properties_changed(self, modified, actions):
         "update label (and size if label/stockitem have changed)"
         if not modified or "bitmap" in modified and self.widget:
             bmp = self.get_preview_obj_bitmap(self.bitmap)
             self.widget.SetBitmap(bmp)
+            if modified: actions.add("layout")
 
-            self._set_widget_best_size()
-
-        EditStylesMixin.properties_changed(self, modified)
-        ManagedBase.properties_changed(self, modified)
+        EditStylesMixin._properties_changed(self, modified, actions)
+        ManagedBase._properties_changed(self, modified, actions)
 
 
-def builder(parent, sizer, pos, number=[1]):
+def builder(parent, index, bitmap=None):
     "factory function for EditStaticBitmap objects"
-    name = 'bitmap_%s' % number[0]
-    while common.app_tree.has_name(name):
-        number[0] += 1
-        name = 'bitmap_%s' % number[0]
-    bitmap = wx.FileSelector(_("Select the image"))
+    name = parent.toplevel_parent.get_next_contained_name('bitmap_%d')
+    if bitmap is None:
+        bitmap = misc.RelativeFileSelector("Select the image")
+        if bitmap is None: return
     with parent.frozen():
-        static_bitmap = EditStaticBitmap(name, parent, wx.NewId(), bitmap, sizer, pos)
-        static_bitmap.properties["style"].set_to_default()
-        static_bitmap.check_defaults()
-        node = Node(static_bitmap)
-        static_bitmap.node = node
-        if parent.widget: static_bitmap.create()
-    common.app_tree.insert(node, sizer.node, pos-1)
+        editor = EditStaticBitmap(name, parent, index, bitmap)
+        editor.properties["style"].set_to_default()
+        editor.check_defaults()
+        if parent.widget: editor.create()
+    return editor
 
 
-def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
+def xml_builder(parser, base, name, parent, index):
     "factory to build EditStaticBitmap objects from a XML file"
-    from xml_parse import XmlParsingError
-    try:
-        label = attrs['name']
-    except KeyError:
-        raise XmlParsingError(_("'name' attribute missing"))
-    if sizer is None or sizeritem is None:
-        raise XmlParsingError(_("sizer or sizeritem object cannot be None"))
-    bitmap = EditStaticBitmap(label, parent, wx.NewId(), '', sizer, pos)
-    #sizer.set_item(bitmap.pos, proportion=sizeritem.proportion, span=sizeritem.span, flag=sizeritem.flag, border=sizeritem.border)
-    node = Node(bitmap)
-    bitmap.node = node
-    if pos is None:
-        common.app_tree.add(node, sizer.node)
-    else:
-        common.app_tree.insert(node, sizer.node, pos-1)
-    return bitmap
+    return EditStaticBitmap(name, parent, index, '')
 
 
 def initialize():
     "initialization function for the module: returns a wx.BitmapButton to be added to the main palette"
+    common.widget_classes['EditStaticBitmap'] = EditStaticBitmap
     common.widgets['EditStaticBitmap'] = builder
     common.widgets_from_xml['EditStaticBitmap'] = xml_builder
 
-    return common.make_object_button('EditStaticBitmap', 'static_bitmap.xpm')
+    return common.make_object_button('EditStaticBitmap', 'static_bitmap.png')
