@@ -24,7 +24,7 @@ class CppStaticTextGenerator(wcodegen.CppWidgetCodeWriter):
     def get_more_properties_code(self, obj):
         ret = []
         if obj.wrap>0:
-            ret.append( '%s->Wrap(%d);\n'%(obj.name, obj.wrap) )
+            ret.append( '%s.Wrap(%d);\n'%(self.tmpl_dict['name'], obj.wrap) )
         return ret
 
 
